#
# spxmanage configuration file for the admin vhost
#
# this file is included from the <VirtualHost> declarations for the
# admin virtual host
#

DocumentRoot /usr/share/spxmanage/www

# The aliases and handlers to redirect the requests to the FastCGI servers
Alias /.fcgi-bin/spxmanage-admin-php /usr/lib/fcgi-bin/spxmanage-admin-php
Action spxmanage-admin-php /.fcgi-bin/spxmanage-admin-php
Alias /.fcgi-bin/spxmanage-www-php /usr/lib/fcgi-bin/spxmanage-www-php
Action spxmanage-www-php /.fcgi-bin/spxmanage-www-php

# Access to logs via web interface
Alias /log/ /srv/raperca/log/

# This is the boot indicator, allows to detect the end of reboot
Alias /boot-id /var/cache/spxmanage/www/docs/boot-id

Alias /content/ /srv/raperca/content/
# Some WebDAV clients always chop off the trailing / in the initial request
# so we need to provide a redirect for them to work correctly. Note that
# redirect directives are always evaluated before alias directives.
RedirectMatch permanent ^/content$ /content/

Alias /capture/ /srv/raperca/capture/

# The official path for the RPC server is /rpc, so that we
# may move away from php if we desire.
Alias /rpc /usr/share/spxmanage/www/rpc.php

# Also to retrive the XML based info the official path is /info,
# so that we may move away from PHP if we desire.
Alias /info /usr/share/spxmanage/www/info.php

# Aliases for the snapshot / monitoring location
Alias /snapshot/info /usr/share/spxmanage/www-snapshot/info.php
Alias /snapshot/snapshot /usr/share/spxmanage/www-snapshot/snapshot.php
Alias /snapshot/ /usr/share/spxmanage/www-snapshot/

# To retrive the snapshot the official path is /snapshot,
# so that we may move away from PHP if we desire.
Alias /snapshot /usr/share/spxmanage/www/snapshot.php

# Place the spxjslibs space in the admin server
# Note that spxjslibs are not necessarily installed;
# as aliases in the main server config are inherited by
# *all* vservers and there can only be a VirtualHost
# section for each vserver in the config we need to
# define this here.
Alias /spxjslibs /usr/share/spxjslibs/libs

# Override the DAV lock DB to use our own
DavLockDB /var/cache/spxmanage/dav/lockdb

# Unconditionally set the which_vhost environment variable
# so that other config files can do per-vhost conditional
# processing reliably (SetEnv cannot be used because it
# occurs late in request processing)
SetEnvIf Request_URI ^ which_vhost=main

# Internet Explorer does not display text/plain MIME type inline, or
# at least not for URLs ending in .log or .log.gz and it starts an
# external viewer based on URL extension and not the MIME type.
# The .log.gz files (compressed) are served with "Content-Type text/plain"
# and "Content-Encoding gzip", which is the right way. However, because
# of the above, IE will try to open it with an application like
# WinZip although it is a plain text file (because the browser has to
# undo the Content-Encoding). The rewrite rules below work around this
# by making IE access the .log.gz files via .log URLs while still being
# sent using "Content-Encoding gzip".

RewriteEngine on

# When executing requests via fcgi an internal URL /.fcgi-bin/ is
# generated, stop any rewriting on that early on so as to avoid
# bogus matches in subsequent rewrite rules
RewriteRule ^/.fcgi-bin/ - [L]
    
# Forbid all direct access to admin.php, this script should not be
# accessed externally, it is instead accessed via a rewrite rule.
RewriteRule ^/admin.php - [F,L,NS]

# Map a .log URL in the logs folder to the .log.gz file if the .log
# file does not exist but the .log.gz file does. Note that this rule
# cannot be in a <Directory> not .htaccess section because it is
# otherwise re-injected into URL processing and would match the rule
# below, causing an infinite number of redirects.
RewriteCond /srv/raperca/$1 !-f
RewriteCond /srv/raperca/$1.gz -f
RewriteRule ^/(log/.*\.log)$		/srv/raperca/$1.gz [NS]

# For IE redirect all accesses to exising .log.gz files to a .log URL
# so that IE sees an extension that matches the MIME type
RewriteCond %{HTTP_USER_AGENT} "\bMSIE [0-9]"
RewriteCond /srv/raperca/$1.gz -f
RewriteRule ^/(log/.*\.log)\.gz$	$1 [R,L,NS]

# Some users mistakenly type just the device hostname on the URL bar
# of HMD to access the content (WebDAV) server, instead of properly
# specifying the WebDAV server on port 81 below. To help these users
# we redirect them to the proper server when receiving a PROPFIND
# (the only WebDAV specific request which has no side-effects).
# We need to handle the case where the host header has a port number
# in it, although normally it will not. We also handle literal IPv4 and
# IPv6 addresses in addition to host names. The result is not
# guaranteed to work when accessed via a reverse proxy or via a
# port mapping device, but at least it works in the general case and
# provides a clue otherwise.
RewriteCond %{REQUEST_METHOD} =PROPFIND
RewriteCond %{HTTPS} =off
RewriteCond %{REQUEST_URI} !^/content(?:/|$)
RewriteCond %{HTTP_HOST} ^(\[[^]]+\]|[^][:]+)(?::[0-9]+|$) [NV]
RewriteRule ^((?!/content).*)$ http://%1:81$1 [R=permanent,L,NS]

# Allow LOCK requests under the DAV enabled part to existing files only,
# see analogous definition under the content virtual server for rationale.
RewriteCond %{REQUEST_METHOD} =LOCK
RewriteCond /srv/raperca%{REQUEST_URI} !-f
RewriteRule ^/content(?:/|$) - [F,NS]

# Accesses to the controlcenter module need to be handled by the
# admin PHP instance. The PT flag is needed for the CGI handler to
# correctly process the new URI as otherwise it uses the original
# URI.
RewriteRule ^(/controlcenter/.*) /admin.php$1 [L,NS,H=spxmanage-admin-php,PT]

# Make the info.php, rpc.php, snapshot.php, report.php run via the admin
# handler.  The PT flag is not necessary since these URIs already
# point to the script or their alias.
RewriteRule ^/(info|rpc|snapshot|report)(?:$|/|\.php) /usr/share/spxmanage/www/$1.php [NS,H=spxmanage-admin-php]

# Bypass PHP script and serve files directly
RewriteCond /srv/raperca/interface/public/$1 -f
RewriteRule ^/api/(.*) /srv/raperca/interface/public/$1 [NS]

# Bypass PHP script and serve files directly for the apps
RewriteCond /srv/raperca/interface/public/apps/$1/interface/$2 -f
RewriteRule ^/assets/apps/edit/(\w*)/(.*) /srv/raperca/interface/public/apps/$1/interface/$2 [NS]

# Send all URIs corresponding to modules or controllers of
# index.php to index.php, also send a bare / or empty URI-path to
# index.php. The PT flag is needed for the CGI handler to
# correctly process the new URI as otherwise it uses the original
# URI.
RewriteRule ^(/(?:api|assets|main|error|user)/.*|/|)$ /index.php$1 [L,NS,PT]

# Support for transparent authentication to content and scripts to
# not require HTTP authentication for clients already
# authenticated with the web interface via forms.
#
# The interface authentication works via PHP session with the
# "uisess" cookie. On login the interface creates the auth_$sessid
# file which it removes on logout. The session storage file
# (sess_$sessid) exists even for non logged in users.
# This part just sets the "has_uiauth" environment variable
# which is then checked in the directory access rules to
# bypass password authentication.
# Note that PHP session IDs for the file handler allows all
# alphanumeric characters, the comma and the minus.
RewriteCond %{HTTP_COOKIE} 'uisess="?([-,a-zA-Z0-9]+)"?' [NV]
RewriteCond /var/cache/php/session/auth_%1 -f
RewriteCond /var/cache/php/session/sess_%1 -s
RewriteRule ^ - [E=has_uiauth]

# Likewise for the admin role, the "has_admin_uiauth" environment
# variable is set.
RewriteCond %{HTTP_COOKIE} 'uisess="?([-,a-zA-Z0-9]+)"?' [NV]
RewriteCond /var/cache/php/session/auth_admin_%1 -f
RewriteCond /var/cache/php/session/sess_%1 -s
RewriteRule ^ - [E=has_admin_uiauth]

# Likewise for the content role, the "has_content_uiauth" environment
# variable is set.
RewriteCond %{HTTP_COOKIE} 'uisess="?([-,a-zA-Z0-9]+)"?' [NV]
RewriteCond /var/cache/php/session/auth_content_%1 -f
RewriteCond /var/cache/php/session/sess_%1 -s
RewriteRule ^ - [E=has_content_uiauth]

# Likewise for the monitoring role, the "has_monitoring_uiauth"
# environment variable is set.
RewriteCond %{HTTP_COOKIE} 'uisess="?([-,a-zA-Z0-9]+)"?' [NV]
RewriteCond /var/cache/php/session/auth_monitoring_%1 -f
RewriteCond /var/cache/php/session/sess_%1 -s
RewriteRule ^ - [E=has_monitoring_uiauth]

# Allow all rewrites from the main server definition to apply
# to this pseudo-virtual server
RewriteOptions Inherit

# Use normal request log file but with extended format
CustomLog /var/log/apache2/access_log common_extended
