#!/bin/sh
#
# Sets the hostname and apache header to the serial number of the device
# prepending the prefix from the resources to the hostname

RCINIT=/usr/share/resources/default/init/main
RCRECOVERY=/usr/share/resources/default/recovery/main

# Load resources
[ -f "$RCINIT" ] && . "$RCINIT"
[ -f "$RCRECOVERY" ] && . "$RCRECOVERY"

# Source the init script functions
. /etc/init.d/functions

case "$1" in
    start|restart)
    if [ ! -s /etc/hostname -o ! -s /etc/apache2/conf.d/30-spxserial.conf ]; then
	sn="$(sed -ne '/^Serial\s\+/{s,.*:\s\+,,p}' /proc/cpuinfo)"
	if [ $? -ne 0 ]; then
	    echo "failed recovering serial number from /proc/cpuinfo"
	    failure
	    exit 1
	fi
	hw="$(sed -ne '/^Hardware\s\+/{s,.*:\s\+,,p}' /proc/cpuinfo)"
	if [ $? -ne 0 ]; then
	    echo "failed recovering hardware type from /proc/cpuinfo"
	    failure
	    exit 1
	fi
	hw="$hw""$rc_hwtype_suffix"
    else
	exit 0
    fi
    if [ -z "$sn" -o "$sn" = "000000000000" -o "$sn" = "0000000000000000" ]; then
	echo "No serial number, using eth0 MAC address as fallback."
	sn="$(sed -e 's/://g' /sys/class/net/eth0/address)"
	if [ -z "$sn" ]; then
	    echo -n "No eth0 MAC address"
	    exit 0
	fi
    fi
    if [ ! -s /etc/hostname ]; then
	echo -n "Setting the hostname to serial number "
	echo "${rc_host_prefix}${sn}" > /etc/hostname
	success
	echo
    fi
    if [ ! -s /etc/apache2/conf.d/30-spxserial.conf ]; then
	echo -n "Setting apache httpd server name and serial number / hardware headers "
	sed -e "s/@@servername@@/${rc_host_prefix}${sn}.local/" \
	    -e "s/@@serial@@/$sn/;s/@@hardware@@/$hw/;" \
	    /etc/apache2/conf.d/30-spxserial.conf.tmpl > /etc/apache2/conf.d/30-spxserial.conf
	success
	echo
    fi
    ;;
    stop|reload|force-reload)
    ;;
esac



