
// Create a class
/*
function obj( params ) {
    this.x = params.x;    
}
obj.prototype = {
    funct : function( params ) {
        this.x;
    },
    attr: 5
};

function o2( params ) {
}
o2.prototype = new obj();
$.extend( o2.prototype, {
    f2: function() {},  
    m2: 2
});
*/

function getValuePath( obj, path ) {
    var parts = path.split('.');
    for (var i=0; i<parts.length && obj !== undefined; i++) {
        obj = ko.unwrap( obj[parts[i]] );
    }
    return obj;
}
function setValuePath( obj, value, path ) {
    var parts = path.split('.');
    var i, tmp;
    for(i = 0; i < parts.length; i++) {
        tmp = obj[parts[i]];
        if( i === parts.length - 1) {
            if ( ko.isObservable( obj[parts[i]] ) ){
                tmp = obj[parts[i]]( value );
            } else {
                if ( value === undefined ) {
                    delete obj[parts[i]];                    
                    tmp = undefined;
                } else {
                    tmp = obj[parts[i]] = value;                    
                }
            }
        } else if(tmp === undefined) {
            tmp = obj[parts[i]] = {};
        }
        obj = tmp;
    }
    return obj;
}


// KO Model View

function SetViewModel( args ) { 
    var construct = args.construct || function( item ) { return item; };
    var size = args.size || "auto";
    var max = args.max || null;
    var setupMax = max;
    var maxItems = args.maxItems || 12;
    var maxCols = args.maxCols || 8;
    var maxRows = args.maxRows || 8;
    
    var debug = args.debug || false;
    
    var self = this;
    self.pageSetup = {
        single:     {
            class: "col-sm-" + maxCols,
            items: 1
        },
        large:      {
            class: "col-sm-" + maxCols/2,
            items: maxItems/4*maxRows/4
        },
        medium:     {
            class: "col-sm-" + maxCols/4,
            items: maxItems/2*maxRows/2
        },
        small:      {
            class: "col-sm-" + Math.floor(maxCols/8),
            items: maxItems*maxRows
        }
    };
    self.items = ko.observableArray()
                    .extend({ rateLimit: 0 }); 
    
    self.length = function ( ) {
        return this.items().length;
    };
    self.item = function ( i ) {
        return this.items()[i];
    };
    self.append = function ( data ) {
        for (var i = 0; i < data.length; i++) {
            self.add( data[i] );
        }                    
    };
    self.refresh = function ( data ) {
        self.clear();
        self.append( data );
    };
    
    
    self.keywordsList = ko.observableArray( args.keywordsList || ['all','divider'] );
    function updateKeywords( data ) {
        for ( var i=0; i< data.length; i++ ) {
            if ( self.keywordsList.indexOf(data[i])===-1 )
                self.keywordsList.push( data[i] );
        }
    }
    
    self.prepare = function ( data ) {
        var media = construct( data );

        if ( media.keywords && media.keywords.subscribe ){
            media.keywords.subscribe( updateKeywords );
            updateKeywords( media.keywords() );
        }
        if ( media.familly ){
            if ( self.keywordsList.indexOf(media.familly) === -1 )
                self.keywordsList.unshift( media.familly );
        }
        if ( !media.selected ) {
            media.selected = ko.observable( false );
        }
        if ( !media.select ) {
            media.select = function( force ) {
                self.selectMedia( this, force );               
            };
        }
        return media;
    };
    self.unshift = function ( data ) {
        var media = self.prepare( data );
        self.items.unshift( media );        
        return media;
    };
    self.push = function ( data ) {
        var media = self.prepare( data );
        self.items.push( media );        
        return media;
    };
    // for backward compatibility
    self.add = function ( data ) {
        return self.push( data );
    };
    self.del = function ( media ) {
        return self.items.remove( media );
    };
    self.delIndex = function ( idx ) {
        return self.items.splice( idx, 1 );
    };
    self.clear = function ( ) {
        self.items.removeAll( );
    };
    
    // selection
    self.selection = ko.observableArray();
    self.selectionIdx = ko.observableArray();
    self.isSelectedMedia = function ( media ) {
        if ( !media )
            return false;
        return media.selected();
        //return self.selection.indexOf( media )!==-1;
    };
    self.isSelectedIndex = function ( idx ) {
        return self.items()[ idx ].selected();
        //return self.selectionIdx.indexOf( idx )!==-1;
    };
    self.selectionLength = function ( ) {
        return self.selection().length;
    };
    self.selectionItem = function ( idx ) {
        if ( idx < this.selection().length )
            return self.selection()[idx];
        else
            return null;
    };
    self.selectionClear = function ( ) {
        for( var i=0; i< self.items().length; i++ ){
            if ( self.items()[i].selected )
                self.items()[i].selected( false );
        }
        if ( self.selection().length>0 ){
            self.selection.removeAll();  
            self.selectionIdx.removeAll();  
        }
    };
    self.selectAll = function () {
        if ( max !== undefined && max !== null )
            return;
        for( var i=0; i< self.items().length; i++ ){
            if ( !self.items()[i].selected() ) {
                self.selectIndex( i, true );
            }
        }
    };
    
    self.selectMedia = function ( media, force ) {
        var sel_idx = self.items.indexOf( media );
        if ( sel_idx<0 )
            return;
        self.selectIndex( sel_idx, force );
    };
    self.selectIndex = function ( idx, force ) {
        var media = self.items()[idx];
        if ( !media ) 
            return;
        
        if ( !self.isSelectedMedia( media ) && force!==false ){
            // add
            if ( max === 1 ) {
                if ( self.selection().length>0 ) 
                    self.selection()[0].selected( false );
                self.selection( [ media] );
                self.selectionIdx( [ idx] );
            } else {
                if ( max && self.items().length === max ){
                    self.selection.pop().selected( false );
                    self.selectionIdx.pop();
                }
                self.selection.push( media );
                self.selectionIdx.push( idx );
            }
            media.selected( true );            
            
        } else if ( force!==true ) {
            // remove
            self.selection.remove( media );
            self.selectionIdx.remove( idx );
            media.selected( false );            
        }
    };
    
    
    self.popSelected = function ( action ) {
        self.actionSelected( action );
        self.selectionClear();                
    };
    self.actionSelected = function ( action ) {
        var sel = [];
        for( var i=0; i<self.selection().length; i++ ){
            sel.push( { media: self.selection()[i], idx: self.selectionIdx()[i] } );
        }
        while ( sel.length>0 ){
            var data = sel.pop();
            action( data.media, data.idx );
        }                
    };
    
    self.read = function ( name, default_value, mixed_value ) {
        var val = null;
        var has_val = false;
        for ( var i=0; i<self.selection().length; i++ ){
            var cur_val = getValuePath(self.selection()[i], name);
            if ( !has_val ){
                val = cur_val;
                has_val = true;
            } else if ( val !== cur_val )
                val = mixed_value;
        }
        if ( val === undefined )
            val = default_value;
        return val;
    };
    self.write = function ( name, value, removeEmpty ) {
        if ( removeEmpty && value==="" )
            value = undefined;
        for ( var i=0; i<self.selection().length; i++ ){
            setValuePath(self.selection()[i], value, name);
        }
    };
    
    self.selectionItemSize = ko.computed(function() {
        if ( self.selectionLength() === 1 )
            return "single";        
        else if ( self.selectionLength() <= 4 )
            return "large";        
        else 
            return "medium";        
        
    });
    self.selectionItemClass = ko.computed(function() {
        return "spx-media-" + self.selectionItemSize();        
    });
    self.selectionColClass = ko.computed(function() {
        if ( self.selectionLength() === 1 )
            return "col-sm-12";        
        else if ( self.selectionLength() <= 4 )
            return "col-sm-6";        
        else
            return "col-sm-3";                
    });
    // filtering 
    self.keyword = ko.observable( args.keyword );
    self.search = ko.observable( args.search );    
    self.sort = ko.observable( args.sort );
    self.filterItems = function( familly ) {
        var localSet;
        if ( familly || self.keyword() || self.search() ) {
            localSet = ko.utils.arrayFilter( self.items(), function( item ) {
                if ( familly && ko.unwrap(item.familly) !== familly )
                    return false;
                if ( self.keyword() && self.keyword() !== 'all' && 
                        ko.unwrap(item.familly) !== self.keyword() && 
                        ko.utils.arrayIndexOf(ko.unwrap(item.keywords), self.keyword()) === -1 )
                    return false;
                if ( self.search() && ko.unwrap(item.name).indexOf( self.search()) === -1 )
                    return false;                    
                return true;
            } );
        } else {
            localSet = self.items();
        }
        if ( self.sort() && localSet.length>0 && localSet[0][self.sort()] !== undefined ) {
            localSet.sort( function( a,b ) {
                return a[self.sort()]>b[self.sort()];
            });            
        }
        
        return localSet;
    }
    self.filteredItems = ko.computed( function() {
        return self.filterItems( );
    });
        
    // size handling
    self.defaultSize = ko.observable( size );
    self.itemSize = ko.computed(function() {
        var new_size = "small";
        var items = self.filteredItems();
        if ( self.defaultSize() && self.defaultSize()!=='auto' )
            new_size = self.defaultSize();
        else if ( items.length===0 )
            new_size = "medium";
        else {
            for ( var p in self.pageSetup ) {
                if ( items.length<self.pageSetup[p].items ){
                    new_size = p;                       
                    break;
                }
            }
        }       
        return new_size;
    });
    var old_size = 'small';
    self.itemSize.subscribe( function( new_size ) {
        if ( old_size === new_size ) 
            return;
        var idx;
        var page = null;
        if ( self.selectionIdx().length>0 ){
            idx = self.selectionIdx()[0];            
        } else {            
            page = Math.floor( self.currentPage()*self.pageSetup[old_size].items/self.pageSetup[new_size].items );
            idx = 0;
        }
        setTimeout(function() {
            if ( page === null )
                page = self.pageForIndex( idx );
            self.currentPage( page );
        }, 0 ); 

        self.selectionClear();
        if ( self.defaultSize()==='single' ){
            self.selectIndex( idx, true );
            max = 1;
        } else {
            max = setupMax;
        }
        old_size = new_size;
        
        var nbMedia = self.pageSetup[new_size].items;
        var nbPages = Math.ceil( self.filteredItems().length/nbMedia );
        if ( nbPages<1 )
            nbPages = 1;
        self.nbPages( nbPages );
    });
    
    self.itemClass = ko.computed(function() {
        return "spx-media-" + self.itemSize();        
    });   
    
    // pages
    self.nbPages = ko.observable( 1 );
    self.filteredItems.subscribe( function( newSet ) { 
        
        var nbMedia = self.pageSetup[self.itemSize()].items;
        var nbPages = Math.ceil( newSet.length/nbMedia );
        if ( nbPages<1 )
            nbPages = 1;
        self.nbPages( nbPages );
        
        var i;
        var removedItems = false;
        var sel = self.selection();
        for ( i=0; i<sel.length; i++ ){
            var media = sel[i];
            if ( newSet.indexOf( media ) === -1 ){
                self.selectMedia( media, false );
                removedItems = true;
            }
        }     
        if ( self.defaultSize()==='single' ) {
            var newPage = -1;
            if ( !removedItems && self.selection().length > 0 )
                newPage = newSet.indexOf( self.selection()[0] );
            if ( newPage < 0 ){
                newPage = 0;
                if ( newSet.length > 0 )
                    self.selectMedia( newSet[0], true );
            }         
            self.currentPage( newPage );            
        } else {
            if ( removedItems || self.selection().length === 0 )
                self.currentPage( 0 );
        }
        
    });
    
    self.currentPage = ko.observable( 0 );
    self.currentPage.subscribe( function( idx ) {        
        if ( self.defaultSize()==='single' ){
            var media = self.filteredItems()[idx];
            self.selectMedia( media, true );            
        } else 
            self.selectionClear();    
    });
    
    
    self.pageData = function( page ){
        if ( page===undefined  )
            page = self.currentPage();
        
        var nb_media = self.pageSetup[self.itemSize()].items;
        var offset = page * nb_media;
        var result = [];
        
        var localSet = self.filteredItems();
        
        for ( var i=offset; i< offset+nb_media && i<localSet.length; i++ )
            result.push( localSet[i] ) ;
        return result;
    };
    self.pageForIndex = function( idx ){
        var data = self.items()[idx];
        var localSet = self.filteredItems();
        var newIdx = localSet.indexOf( data );
        if ( newIdx<0 )
            return 0;
        var nbMedia = self.pageSetup[self.itemSize()].items;
        return Math.floor( newIdx/nbMedia );
    };
    self.colClass = ko.computed(function() {
        return self.pageSetup[self.itemSize()].class;
    });
    
    self.currentPageData = ko.computed(function() {        
        return self.pageData( self.currentPage() );
    });
            
};

function MediaViewModel( media ) {
    var self = this;
    //self.media = media;
    self.id = media.id;
    self.manage = media.manage;
    self.name = ko.observable( media.name );
    
    var desc = "";
    if ( media.type ){
        if ( media.type.indexOf('/')>0 )
            self.familly = media.type.substr(0, media.type.indexOf('/') );        
        else
            self.familly = media.type;        
    } else
        self.familly = 'unknown';
    self.type = media.type;
    if ( self.familly==='audio' || self.familly==='video' ) {
        self.dur = media.dur;
        desc += "\nduration: " + self.dur;
    } else
        self.dur = false;
        
    if ( self.familly==='image' || self.familly==='video' ) {
        self.dimension = media.dimension;
        self.aspectRatio = media.aspectRatio;
        desc += "\ndimension: " + self.dimension;
    } else
        self.dimension = false;
    
    self.keywords = ko.observableArray( media.keywords );        
    self.desc = ko.observable( desc );
    self.snapshots = spxapi.resources.get( self.id ).snapshots;
}
function MediaList( args ) {
    var self = this;
    
    self.set = new SetViewModel( {
        construct: function( data ) { 
            return new MediaViewModel( data );             
        }, 
        size: args.size,
        maxItems: args.maxItems,
        maxCols: args.maxCols,
        maxRows: args.maxRows,
        keyword: args.keyword,
        keywordsList: args.keywordsList || ['image', 'video','divider']
    });
    
    self.resources = args.resources || ['media'];
    self.refresh = function () {
        self.set.clear();
        for( var i=0; i<self.resources.length; i++ ){            
            spxapi[self.resources[i]].list().done( function( data ) {
                self.set.append( data );            
            });
        }
    };
    self.delIndex = function( idx ) {
        self.del( self.set.item( idx ) );
    };
    
    self.del = function ( media, force ) {
        spxapi.media.del( media.manage, force ).done( function( data, textStatus, jqXHR ) { 
            if ( jqXHR.status===202 ) {
                var txt = "<p>"+media.name()+" is used by:<ul>";
                for (var i=0; i< data.length; i++ ) {
                    txt += "<li>"+data[i].type + ": "+data[i].name+"</li>";
                }
                txt += "</ul></p>";
                
                modalDialog( {
                    title: media.name() + " in use",
                    body: txt,
                    buttons : [
                        {  
                            name: "Delete anyway",
                            action: function() {
                                self.del( media, true );
                            }
                        },
                        { name: "Cancel" }
                    ]
                } ).modal('show');
            } else
                self.set.del(media);
        });
    };
    
    self.find = function( id ) {
        for( var i=0; i<self.set.length(); i++ ){
            var item = self.set.item(i);
            if ( item.id === id )
                return item;
        }   
    };
    
    self.refresh();
    
    return self;
}

function WidgetViewModel( widget ) {
    var self = this;
    //self.media = media;
    if ( widget.id ) {
        self.id = widget.id;
        self.manage = widget.manage;
        self.widget = widget;
        self.keywords = widget.keywords;
    } else {
        // this is a media playlist
        self.id = null;                
    }
    if ( self.keywords===undefined || self.keywords.length===0 )
        self.keywords = ['default'];
    self.name = ko.computed( function() {
        return widget.name;        
    });
    self.desc = ko.computed( function() {
        return widget.desc;        
    });
    
    
    self.snapshots = spxapi.resources.get( self.id ).snapshots;
}
function WidgetList( args ) {
    var self = this;

    self.keywordsList = ko.observableArray( [ 'default' ]);

    self.set = new SetViewModel( {
        construct: function( data ) {
            return new WidgetViewModel( data );             
        }, 
        size: args.size
    });
    
    self.refresh = function () {
        spxapi.apps.list().done( function( data ) {
            // insert a placeolder for the 'playlist widget' 
            if ( args.placeolder && args.placeolder==='playlist' ){
                data.unshift( {
                    id: null,
                    name: "New media layer",
                    desc: "This layer let you add media to the playout"
                });
            }
            self.set.refresh( data );            
        });
    };
    
    self.find = function( id ) {
        for( var i=0; i<self.set.length(); i++ ){
            var item = self.set.item(i);
            if ( item.id === id )
                return item;
        }   
    };
    
    self.refresh();
    
    return self;
}

function MediaItem( media ) {
    var self = this;
    //self.media = media;
    self.id = media.id;    
    self.type = media.type;
    self.dur = ko.observable( media.dur );
    self.schedule = { 
        beginDate: ko.observable(),
        endDate: ko.observable()
    };
    if ( media.schedule ){
        if ( media.schedule.beginDate){  
            self.schedule.beginDate(
                    media.schedule.beginDate.substr( 0, media.schedule.beginDate.indexOf("T") ) );
        } else
            
        if ( media.schedule.endDate){  
            self.schedule.endDate(
                    media.schedule.endDate.substr( 0, media.schedule.endDate.indexOf("T") ) );
        }
        // should handle zones
    }
    self.getSchedule = function () {
        var schedule = { };
        if ( self.schedule.beginDate() )
            schedule.beginDate = self.schedule.beginDate() + "T00:00:00";
        if ( self.schedule.endDate())
            schedule.endDate = self.schedule.endDate() + "T23:59:59.999";
        
        return schedule;
    };
    self.snapshots = spxapi.resources.get( self.id ).snapshots;
    if ( media.type === 'slide' )
        fixApps( media, self, 'slide' );
}
function computedSelected( self, name, default_value, mixed_value ){
    return {
        read: function () {
            return self.set.read(name, default_value, mixed_value);
        },
        write: function ( value ) {
            if ( value==="" )
                value = undefined;
            return self.set.write(name, value);
        },
        owner: this
    };
}


function Playlist( args ) {
    var self = this;
    
    self.set = new SetViewModel( {
        construct: function( mediaItem ) { return new MediaItem( mediaItem ); },
        size: args.size
    } );
    
    
    
    self.isNew = ko.observable( true );
    
    self.fromJSON = function( playlist ) {
        if ( playlist.name )
            self.name( ko.unwrap(playlist.name) );
        if ( playlist.options ) {
            self.options.defaultDur( ko.unwrap(playlist.options.defaultDur) );
            self.options.shuffle( ko.unwrap(playlist.options.shuffle) );
        } 
        if ( playlist.media ) 
            self.set.refresh( ko.unwrap(playlist.media) );
        
        self.isNew( false );
    };
    
    self.toJSON = function( keepApps ) {
        var params = {
            name : self.name(),
            options : {
                defaultDur: self.options.defaultDur(),
                shuffle: self.options.shuffle()
            },
            media: []
        };
        for (var i=0; i<self.set.length(); i++ ){
            var cur = self.set.items()[i];
            var media = { 
                id: cur.id,
                type: cur.type 
            };
            if ( cur.dur ) 
                media.dur = cur.dur();   
            media.schedule = cur.getSchedule(); 
            if ( cur.type==='slide' ){
                media.query = fixQuery( cur.options() );
                if ( keepApps ) {
                    media.options = cur.options;
                    media.apps = cur.apps;
                    media.desc = cur.desc;
                }
            }            
            params.media.push( media );            
        }
        return params;
    };
    
    
    self.name = ko.observable( args.defaultName || "New playlist" );
    // options
    self.options = {
        defaultDur: ko.observable( 10 ),
        shuffle: ko.observable( false )
    };
        
    self.refresh = function () {
        if ( self.uri )
            spxapi.playlist.get( self.uri ).done( self.fromJSON );
    };
    if ( args.save ){
        self.save = args.save;
    } else {
        self.save = function ( ) {
            var params = self.toJSON();        
            if ( self.isNew() ){ 
                spxapi.playlist.create( params ).done( function() { 
                    self.cancel();
                });
            } else {
                spxapi.playlist.update( self.uri, params ).done( function() { 
                    self.cancel();
                });
            }
        };
    }
    if ( args.cancel ) {
        self.cancel = args.cancel;
    } else {
        self.cancel = function ( ) { };
    }
    self.mediaDragged = function ( item, event, ui ) {
        var mediaType = 'media';
        if ( item.familly === 'playlist' || item.familly === 'slide' )
            mediaType = item.familly;
        
        var media = { 
            id: item.id, 
            type: mediaType,
            snapshots: item.snapshots
        };        
        if ( mediaType==='slide' ) {
            media.options = ko.observableArray().extend({ rateLimit: 0 }); ;
            media.apps = ko.observable();            
            spxapi.slide.get( item.manage ).done( function( slide ) {
                // slide has been loaded, shall we do something about it ?                
                fixApps( slide, media ); 
            } ); 
        }
        return self.set.prepare( media );
    };
        
    
    if ( args.id ){
        self.uri = spxapi.playlist.uri + args.id + "/";
        self.refresh();
    }
    if ( args.playlist ){
        // do this once the object has been created
        setTimeout(function() {
            self.fromJSON( args.playlist );            
        }, 0 );
    }
    
    self.breadcrumbs = ko.observableArray().extend({ rateLimit: 0 });
    
    self.editing = ko.observable( false );
    self.mediaOptions = ko.observableArray();
    
    self.startEditOptions = function() {
        self.editing( true );
        self.breadcrumbs()[self.breadcrumbs().length-1].active( false );
        var items = "";
        var sel = self.set.selectionIdx();
        for ( var i=0; i<sel.length; i++ ){
            items = items + "#" + (sel[i]+1) + ", ";
        }
        if ( sel.length > 1 )
            items = "Items: " + items;
        else
            items = "Item: " + items;
        self.breadcrumbs.push({
            name: items.substr(0, items.length-2),
            active: ko.observable( true ),
            click: false
        });
        self.mediaOptions( [
            { name: 'Dur', where:'dur', type : 'string', 
                value: ko.observable( self.set.read( 'dur', 'default', '' ) ) },
            { name: 'Start', where:'schedule.beginDate', type : 'datetime', 
                value: ko.observable( self.set.read( 'schedule.beginDate', null, null ) ) },
            { name: 'End', where:'schedule.endDate', type : 'datetime', 
                value: ko.observable( self.set.read( 'schedule.endDate', null, null ) ) }         
        ]);        
    };
    self.saveEditOptions = function() {
        var opt = self.mediaOptions();
        for( var i=0; i<opt.length; i++ ){
            self.set.write( opt[i].where, opt[i].value(), true ); 
        }
        
        self.cancelEditOptions( );
    };
    self.cancelEditOptions = function() {
        if ( self.editing() ) {
            self.editing( false );    
            self.breadcrumbs.pop();
            self.breadcrumbs()[self.breadcrumbs().length-1].active( true );            
        }
    };
    
    self.breadcrumbs( [ { 
        name: self.name,
        active: ko.observable( true ),
        click: self.cancelEditOptions
    } ]);
    
    return self;
}

var background_drop = null;
function fixChoices( option ) {    
    if ( option.type === 'choice' ){
        if ( !option.names )
            option.names = option.choices;
        option.select = [{
                name: 'default',
                value: ''
            }];
        for (var i=0; i<option.choices.length; i++ ){
            option.select.push({
                name: option.names[i],
                value: option.choices[i]
            });
        }
    }
    return option;
}
function fixApps( src, dst, type ){
    
    if ( type && src.options && ko.isObservable(src.options) && src.apps && ko.isObservable(src.apps)){
        dst.options = src.options;
        dst.apps = src.apps;
        dst.desc = src.desc;
        dst.id = src.id;
        return dst;
    }
    if ( type )
        var apps = ko.unwrap( src[type] );
    else
        var apps = ko.unwrap( src );
    dst.desc = apps.desc;
    dst.id = apps.id;
    
    if ( !dst.options || !ko.isObservable(dst.options) )
        dst.options = ko.observableArray().extend({ rateLimit: 0 }); ;
    if ( apps.options ) {
        for( var q in apps.options ) {
            var option = {
                query: q,
                value: ko.observable(  )
            };
            $.extend(option, apps.options[q]);
            if ( src.query && src.query[q] )
                option.value( src.query[q] );

            dst.options.push( fixChoices( option ) );
        }
    }
    if ( !dst.apps || !ko.isObservable(dst.apps) )
        dst.apps = ko.observable();
    if ( apps.apps ) {
        dst.apps( apps.apps );
        dst.options.push( {
            query: 'apps',
            value: ko.observable( apps.apps.variable ),
            name: "Apps",
            details: apps.apps.name,
            type: 'apps'
        } );
    }
    return dst;
}
  
function fixQuery( options ) {
    var query = {};
    for ( var j=0; j<options.length; j++ ){
        if ( !options[j].value )
            continue;
        var val = ko.unwrap( options[j].value );
        if ( val!=="" && val!==undefined )
            query[options[j].query] = val; 
    }
    return query;
}  
var layerId = 1000;
function LayerItem( layer, size ) {
    var sizes = {
        svg: { width: 1920, height: 1080 },
        html: { width: 759, height: 428 }           
    };
    var self = this;
    self.layerid = layerId++;
    self.size = size || "large";
    self.updatePlaylist = function( playlist ) {
        if ( playlist.name  ) {
            self.playlist.name( playlist.name );
        }
        if ( playlist.media  ) {
            self.playlist.media.removeAll();
            for ( var i=0; i< playlist.media.length; i++ ){
                var media = $.extend({}, playlist.media[i] );
                if ( media.type==='slide' ){                    
                    fixApps( playlist.media[i], media, 'slide' );                    
                }
                self.playlist.media.push( media );
            }
        }
    };
    self.clear = function( ) {
        self.playlist.media.removeAll();        
    };
    
    
    self.type = layer.type || 'playlist';
    self.background = layer.background;
    
    var args;
    if ( self.background ){
        args = {
            top: 0, left: 0,
            width: sizes.svg.width, height: sizes.svg.height
        };
    } else {
        args = layer.args || {
            width: sizes.svg.width/2, height: sizes.svg.height/2
        };
        if ( args.top===undefined )
            args.top = (sizes.svg.height-parseFloat(args.height) )/2;
        if ( args.left===undefined )
            args.left = (sizes.svg.width-parseFloat(args.width) )/2;
    }
    self.args = {
        top: ko.observable( args.top ),
        left: ko.observable( args.left ),
        width: ko.observable( args.width ),
        height: ko.observable( args.height )
    };
    
    if ( self.type === 'playlist' ){
        self.playlist = { 
            name: ko.observable( self.background?'background':'new layer'),
            media: ko.observableArray().extend({ rateLimit: 0 })
        };
        if ( layer.playlist )
            self.updatePlaylist( layer.playlist );        
    }
    self.apps = ko.observable();
    
    if ( self.type === 'widget' ){        
        fixApps( layer, self, 'widget' );        
    }
    self.appsLayer = function(){
        if ( self.type==='widget' )
            return self;
        if ( self.type==='playlist' ){
            if( self.playlist.media().length===1 ) {
                var media = self.playlist.media()[0];
                if ( media.type==='slide' )
                    return media;
            }
        }
        return null;
    };
    
    self.layerOptions = ko.computed( function() {
        var layer = self.appsLayer();
        if ( layer )
            return layer.options();        
        return null;
    });
    self.name = ko.computed({
        read: function() {
            if ( self.type === 'playlist' )
                return self.playlist.name();
            else
                return layer.widget.name;
        },
        write: function( val ) {
            if ( self.type === 'playlist' )
                self.playlist.name( val );
        }
    });
    self.previewLayer = function() {
        var layer = self;
        if ( self.type === 'playlist' ){
            if ( self.playlist.media().length>0 ){
                layer = self.playlist.media()[0];
            } else 
                return null;
        }
        return layer;
    };
    self.preview = ko.computed(function() {
        var id = null;
        var layer = self.previewLayer();
        if ( layer===null )
            return spxapi.resources.get( null ).snapshots[self.size].src();
        if ( layer.type === 'widget' || layer.type === 'slide' ) {
            id = layer.id;
            var snap = spxapi.resources.get( id ).snapshots[self.size];
            if ( snap.elem()==='svg' ) {
                var q = "";
                var s = "?";
                var options = layer.options();
                for ( var i=0; i<options.length; i++ ){
                    if ( options[i].value()!==undefined && options[i].value()!==""){
                        q += s  + encodeURIComponent(options[i].query) 
                                + "=" 
                                + encodeURIComponent(options[i].value());
                        s = "&";
                    }
                }
                return snap.src() + q;
            } else
                return snap.src();
        } else {
            id = layer.id;  
            return spxapi.resources.get( id ).snapshots[self.size].src();
        }       
    });
    self.staticPreview = ko.computed(function() {
        var layer = self.previewLayer();
        if ( layer === null )
            return true;
        if ( layer.type === 'widget' || layer.type === 'slide' ) {
            var id = layer.id;
            return spxapi.resources.get( id ).snapshots[self.size].elem()==='image';
        }
        return true;
        
    });
    self.htmlLeft = ko.computed(function() {
        return self.args.left()/sizes.svg.width*sizes.html.width + "px";
    });
    self.htmlTop = ko.computed(function() {
        return self.args.top()/sizes.svg.height*sizes.html.height + "px";
    });
    self.htmlWidth = ko.computed(function() {
        return self.args.width()/sizes.svg.width*sizes.html.width + "px";
    });
    self.htmlHeight = ko.computed(function() {
        return self.args.height()/sizes.svg.height*sizes.html.height + "px";
    });
    
    self.transform = ko.computed(function() {
        return 'translate('+self.args.left()+','+self.args.top()+')';
    });
    self.addSingleMedia = function( item ) {
        
        var mediaType = 'media';
        if ( item.familly === 'playlist' || item.familly === 'slide' )
            mediaType = item.familly;
        
        var mediaItem = { 
                id: item.id,
                type: mediaType
        };        
        if ( mediaType==='slide' ) {
            mediaItem.options = ko.observableArray().extend({ rateLimit: 0 }); 
            mediaItem.apps = ko.observable( );
            spxapi.slide.get( item.manage ).done( function( slide ) {
                // slide has been loaded, shall we do something about it ?                
                fixApps( slide, mediaItem );        
            } ); 
        }
        self.playlist.media.unshift( mediaItem );            
        self.playlist.media.splice( 1, self.playlist.media().length-1 );
        if ( self.select )
            self.select( true );
    };
    self.drop = function( item, event, ui ) {
        if ( self.background ){
            if ( background_drop===null ){
                background_drop = true;
                // do it at the end
                setTimeout(function() {
                    self.drop( item, event, ui );
                }, 0 ); 
                return true; // drop is fine for the background
            } else if ( background_drop === false ) {
                background_drop = null;
                return true; // this was not for the background
            } else {
                // this is for us
                background_drop = null;
            }
        } else {
            background_drop = false;
        }
        if ( self.type!=='playlist' )
            return false;
        
        if ( self.playlist.media().length <= 1 ){
            self.addSingleMedia( item );
        } else {
            modalDialog( {
                title: "Warning",
                body: "<p>This will replace the content of the set by a single file</p>",
                buttons : [
                    {   
                        name: "Replace",
                        action: function() {
                            self.addSingleMedia( item );
                        }
                    },
                    { name: "Cancel" }
                ]
            } ).modal('show');           
        }
        
        return true;
    };
    self.drag = function( event, ui ) {
        var realTop = Math.round( ui.position.top*sizes.svg.height/sizes.html.height );
        var realLeft = Math.round( ui.position.left*sizes.svg.width/sizes.html.width );
        if ( realTop<0 )
            realTop = 0;
        if ( realLeft<0 )
            realLeft = 0;
        if ( realTop>sizes.svg.height-self.args.height() )
            realTop=sizes.svg.height-self.args.height();
        if ( realLeft>sizes.svg.width-self.args.width() )
            realLeft=sizes.svg.width-self.args.width();
        
        self.args.left( realLeft );
        self.args.top( realTop );
    };
    self.resize = function( event, ui ) {
        var realTop = Math.round( ui.position.top*sizes.svg.height/sizes.html.height );
        var realLeft = Math.round( ui.position.left*sizes.svg.width/sizes.html.width );
        
        var realHeight = Math.round( ui.size.height*sizes.svg.height/sizes.html.height );
        var realWidth = Math.round( ui.size.width*sizes.svg.width/sizes.html.width );
        
        self.args.left( realLeft );
        self.args.top( realTop );
        
        self.args.height( realHeight );
        self.args.width( realWidth );
    };
}

function Playout( args ) {
    var self = this;
    
    
    self.set = new SetViewModel( {
        construct: function( layerItem ) { return new LayerItem( layerItem, self.size ); },
        max: 1
    });
     
    self.size = args.size || "large";
    
    self.name = ko.observable(  "New playout" );
    // options
    self.options = {
        dur: ko.observable( )      
    };
    
    
    self.refresh = function () {
        if ( self.uri )
            spxapi.playout.get( self.uri ).done( self.fromJSON );
    };
    self.fromJSON = function( playout ) {
        self.name( playout.name );
        
        if ( !playout.options )
            playout.options = { dur: null };
        self.options.dur( playout.options.dur );
        
        if ( !playout.layers )
            playout.layers = [ ];
        if ( playout.layers.length===0 )
            playout.layers.push( {} );
        // layer 0 is special, this is the background and it is always present
        playout.layers[0].background = true;
        self.set.refresh( playout.layers );
    };
    self.toJSON = function () {
        var params = {
            name : self.name(),
            options : {
                dur: self.options.dur()
            },
            layers: []
        };
        for (var i=0; i<self.set.length(); i++ ){
            var cur = self.set.items()[i];
            var layer = {
                type: cur.type,
                args: ko.toJS( cur.args )
            };
            if ( layer.type === 'playlist' ){                
                layer.playlist = {
                    options: ko.toJS( cur.playlist.options ),
                    media: [],
                    name: cur.playlist.name()
                };
                
                // add the media
                for ( var j=0; j<cur.playlist.media().length; j++ ){
                    var media = cur.playlist.media()[j];
                    var mediaItem = {
                        id: media.id,
                        type: media.type                        
                    };
                    if ( media.type === 'slide' ){
                        mediaItem.query = fixQuery( media.options() ); 
                    }
                    layer.playlist.media.push( mediaItem );
                }
            } 
            if ( layer.type === 'widget' ){
                layer.id = ko.unwrap( cur.id );
                layer.query = fixQuery( cur.options() );                
            }
            params.layers.push( layer );
        }
        return params;
    };
    if ( args.save ){
        self.save = args.save;
    } else {
        self.save = function ( ) {
            var params = self.toJSON();
            if ( is_new ){ 
                spxapi.playout.create( params ).done( function() { 
                    self.cancel();
                });
            } else {
                spxapi.playout.update( self.uri, params ).done( function() { 
                    self.cancel();
                });
            }
        };
    }
    if ( args.cancel ) {
        self.cancel = args.cancel;
    } else {
        self.cancel = function ( ) { };
    }
    
    self.addLayer = function ( input ) {
        var layer = input || {};
        self.set.selectionClear();  
        var layer = self.set.add( layer );
        self.set.selectMedia( layer );
        return layer;
    };
    self.mediaDragged = function ( item, event, ui ) {
        var layer = { 
            id: item.id, 
            snapshots: item.snapshots
        };
        return self.set.prepare( layer );
    };
    
    var is_new = true;
    if ( args.id ){
        self.uri = spxapi.playout.uri + args.id + "/";
        is_new = false;
        self.refresh();
    } else {
        self.fromJSON( {
            name: 'New Playout'
        });
    }
    
    self.drop = function( item, event, ui ) {
        ui.position.top = ui.position.top-70;
        if ( item.id ) {
            // this is a widget
            // should notify the user that we are loading the widget
            spxapi.widget.get( item.manage ).done( function( widget ) {
                var layer = {
                    type: 'widget',
                    id: widget.id,
                    widget: widget,                
                    args: {}
                };
                var dim = widget.dimension.split("x");
                layer.args = {
                    width: dim[0],
                    height: dim[1],
                    left: ui.position.left - dim[0]/2,
                    top: ui.position.top - dim[2]/2
                };            
                var where = self.addLayer( layer ); 
                where.drag( event, ui );                
                // widget has been loaded
            } ); 
        } else {
            // new playlist layer
            var dim = [ 1280, 720 ];
            var where = self.addLayer( {
                type: 'playlist',
                args: {
                    width: dim[0],
                    height: dim[1],
                    left: ui.position.left - dim[0]/2,
                    top: ui.position.top - dim[2]/2
                }
            } );
            where.drag( event, ui );
        }
        
    };
    self.breadcrumbs = ko.observableArray().extend({ rateLimit: 0 });
    self.breadcrumbs ( [ { 
            name: self.name,
            active: ko.observable( true ),
            click: false
        } ]);
    
    return self;
}

function dirtyProps(media, isInitiallyDirty) {
    var result = function() {},
        _initialName = ko.observable( ko.toJSON(media.name)),
        _initialKeywords = ko.observable( ko.toJSON(media.keywords)),
        _initialOptions = ko.observable( ko.toJSON(media.options)),
        _isInitiallyDirty = ko.observable(isInitiallyDirty);


    result.isDirty = ko.computed(function() {
        return _isInitiallyDirty() || 
                _initialName() !== ko.toJSON(media.name) || 
                _initialKeywords() !== ko.toJSON(media.keywords) || 
                _initialOptions() !== ko.toJSON(media.options);
    });

    result.reset = function() {
        _initialName(ko.toJSON(media.name));
        _initialKeywords(ko.toJSON(media.keywords));
        _initialOptions(ko.toJSON(media.options));
        _isInitiallyDirty(false);
    };

    return result;
};

function ActiveMedia( args ) {
    var self = this;
    self.media = ko.observable();
    self.newKeyword = ko.observable();
    self.newKeyword.subscribe( function( value ) {
        var media = self.media();
        if ( !media || ! value || value==="" )
            return;
        if ( value && value!=="" && media.keywords.indexOf(value) === -1 ) {
            media.keywords.push( value );            
        }
        setTimeout(function() {
            self.newKeyword("");
        }, 0 );
    });
    self.delKeyword = function( data, ev ) {
        if ( !self.media ) 
            return;
        self.media().keywords.remove( data );
    };
    if ( args.set ){
        args.set.selection.subscribe( function( set ) {
            if ( set.length===1 && args.set.itemSize()==='single' ) {
                // load the activeMedia
                var media = set[0];
                if ( media.dirty===undefined ) {
                    media.dirty = dirtyProps( media, false );
                }
                self.media( media );   
            } else {
                self.media( null );
            }
        });
    }
    self.clean = ko.computed( function() {
        if ( !self.media() )
            return false;
        if ( !self.media().dirty )
            return false;
        var dirty = self.media().dirty.isDirty();
        return !dirty;
    });
    self.clean.extend({ notify: 'always' });
    if ( args.save ){
        self.save = function( data, ev ) {
            $(ev.target).button('loading');
            var media = self.media();
            args.save( media ).done( function( ) {
                $(ev.target).button('reset');
                // using a timeout because 'reset' of the button use a timeout 
                // and we want to make sure
                // the the button is disabled in the end.
                setTimeout(function () {
                    media.dirty.reset();
                }, 0);
                
            } );            
        };
    }
}

function OptionItem( type, value ) {
    var self = this;
    
    if ( type==='media' || type==='playlist' ){
        var data = null;
        if ( value )
            data = JSON.parse( value );
        else 
            data = {};
        if ( type==='media' )
            self.playlist = new Playlist( { playlist: { name: 'media', media: [ data ] } } );
        else
            self.playlist = new Playlist( { playlist: data } );
        self.familly = 'media';
        self.value = ko.computed( function() {
            if ( type==='media' ) {
                if ( self.playlist.set.length()>0 )
                    return JSON.stringify( {
                        id: ko.toJS( self.playlist.set.item(0).id ),
                        type: ko.toJS( self.playlist.set.item(0).type ),
                    });
                else
                    return undefined;
            } else {
                return JSON.stringify( self.playlist.toJSON() );
            }
        });
        self.preview = ko.computed(function() {
            var id = null;
            if ( self.playlist.set.length()>0 )
                id = self.playlist.set.item(0).id;  
            return spxapi.resources.get( id ).snapshots["large"].src();
        });
    } else {
        self.value = ko.observable( value );
        self.familly = 'input';        
    } 
    
    self.drop = function( item, event, ui ) {    
        var mediaType = 'media';
        if ( item.familly === 'playlist' || item.familly === 'slide' )
            mediaType = item.familly;
        
        if ( self.playlist.set.length() <= 1 ){
            self.playlist.set.unshift( { 
                id: item.id,
                type: mediaType
            } );            
            self.playlist.set.items.splice( 1, self.playlist.set.length()-1 );
        } else {
            modalDialog( {
                title: "Warning",
                body: "<p>This will replace the content of the set by a single file</p>",
                buttons : [
                    {   
                        name: "Replace",
                        action: function() {
                            self.playlist.set.unshift( { 
                                id: item.id,
                                type: mediaType
                            } );            
                            self.playlist.set.items.splice( 1, self.playlist.set.length()-1 );
                        }
                    },
                    { name: "Cancel" }
                ]
            } ).modal('show');           
        }
        
        return true;
    };
}

function AppsModel( args ) {
    var self = this;
    
    self.mainMedia = ko.observable( );
    
    self.options = ko.observableArray( ).extend({ rateLimit: 0 });
    self.values = ko.observableArray( ).extend({ rateLimit: 0 }); 
    
    if ( args.manage )
        self.manage = args.manage;
    else if ( args.id )
        self.manage = spxapi.apps.uri + args.id;
    else
        self.manage = null;
    
    
    self.name = ko.observable( );
    self.desc = ko.observable( );
    self.max = ko.observable( );
    self.valueAt = function( row, idx ){
        var row = self.values()[row](); 
        return row[idx].value;
    };
    self.addRow = function ( valueRow ) {
        if ( self.values().length >= self.max() )
            return;
        var row = ko.observableArray( ).extend({ rateLimit: 0 }); 
        for (var i=0; i< self.options().length; i++ ) {
            var cols = self.options()[i].cols;
            var value = valueRow[cols] ;            
            var type = self.options()[i].type;

            var item = new OptionItem(type, value);
            // the main media is the first media we found
            if (!self.mainMedia()
                    && (type === 'media' || type === 'playlist')) {
                item.cols = cols;
                self.mainMedia( item );
            }
            row.push(item);
        }
        self.values.push(row);
    };
    self.refresh = function () {        
        spxapi.apps.get( self.manage ).done( function( data ) {
            self.name( data.name );
            self.desc( data.desc );
            self.max( data.max || 1 );
            
            // work with the array without notifying subscribers
            var options = self.options();
            for ( var cols in data.options ) {
                var option = {
                    cols: cols
                };
                $.extend(option, fixChoices( data.options[cols] ) );
                options.push( option );
            }            
            if ( data.values) {
                for (var i = 0; i < data.values.length; i++) {
                    self.addRow( data.values[i] );                    
                }
            }
            if ( self.values().length === 0 ) {
                // add an empty
                self.addRow( {} );
            }
            // notify the subscribers
            self.options.valueHasMutated();
        });
    };
    self.refresh();
    self.toJS = function( ) {
        var params = {
            name : self.name(),
            values : [ { } ]
        };
        for ( var i=0; i<self.values().length; i++){
            var row = self.values()[i];
            var vals = {};
            for (var j=0; j<self.options().length; j++ ) {
                vals[ self.options()[j].cols ] = row()[j].value();                
            }
            params.values[i] = vals;
        }
        
        if ( self.mainMedia() ) {
            params.values[0][self.mainMedia().cols] = self.mainMedia().value();
        }        
        return params;
    };
    self.apply = function ( ) {
        var params = self.toJS();
        return spxapi.apps.update( self.manage, params );       
    };
    
    if ( args.cancel ) {
        self.cancel = args.cancel;
    } else if ( args.cancelHref) {
        self.cancel = function ( ) { location.href = args.cancelHref; };
    } else {
        self.cancel = function ( ) { };
    }
    
    if ( args.save ){
        self.save = args.save;
    } else {
        self.save = function ( ) {
            self.apply().done( function() {
                self.cancel();
            });
        };
    }    
}