function extendModelFirmware( yiiModel, config ){
    if ( !( 'ipChecked' in yiiModel ) ) {        
        
        function showFirmwareInfo( args ){
            if ( config.infoTarget ){
                args.type = args.type || "info";
                args.body = args.body || args.text || false;
                var cnt = "";
                if ( args.title && args.body ){
                    cnt = '<h4>'+args.title+'</h4><div>'+args.body+'</div>';
                } else if ( args.title ){
                    cnt = args.title;
                } else {
                    cnt = args.body;
                }
                $(config.infoTarget).html('<div class="alert alert-'+args.type+'" role="alert">'+cnt+'</div>');
            } else {
                return spxapi.ui.message( args );
            }
        }
        function clearFirmwareInfo( id ){
            if ( config.infoTarget ){
                $(config.infoTarget).html("");
            } else {
                spxapi.ui.removeMessage( id );
            }
        }
        yiiModel.ipChecked = ko.observable( { executed: false } );
        yiiModel.checkIp = function ( ) {
            yiiModel.ipChecked( { executed: false } );
            $.getJSON( config.urlNetCheck, function( data ){
                data.executed = true;
                yiiModel.ipChecked( data );
            });    
        };
        yiiModel.firmwareSource = ko.observable( config.source || 'base' );
        
        var firmwareUpdateStartedId = null;
        yiiModel.firmwareInfo = ko.observable( { executed: false } );
        yiiModel.firwareInfoTitle = ko.computed( function(){
            var info = yiiModel.firmwareInfo();
            if ( !info.executed ) {
                return 'Updates Check';
            }
            if ( info.reason ) {
                return 'Updates Check Error';
            }
            if ( !info.updates) {
                return "Updates Checked";
            }
            return "Firmware";
        });
        yiiModel.firwareInfoText = ko.computed( function(){
            var info = yiiModel.firmwareInfo();
            if ( !info.executed ) {
                return 'Checking for new firmware version';
            }
            if ( info.reason ) {
                return info.reason;
            }
            if ( !info.updates) {
                return "Firmware is up to date.";        
            }
            return "Updates available.<br/>" + yiiModel.firmwareMsg( info );
        });
        yiiModel.firmwareStatus = ko.observable( { started: false } );
        yiiModel.firmwareStatus.subscribe( function( newValue ) {    
            if ( newValue.reason ) {
                showFirmwareInfo( { 
                    type: 'warning', 
                    title: "Firmware Update Error",
                    sticky: true,
                    text: newValue.reason
                } );
            }
        });
        yiiModel._firmwareUpdateConfig;
        yiiModel.updateCheck = function ( source, forceCheck ) {
            $.getJSON( config.urlFirmware, { updatesource: source, forceCheck: forceCheck }, 
                function( data ){
                    data.executed = true;
                    yiiModel.firmwareInfo( data );
                    yiiModel.firmwareSource( data.source );
                }
            );
        };
        yiiModel.checkFirmware = function ( source, forceCheck, ipCheck ) {
            yiiModel.firmwareInfo( { executed: false } );     
            yiiModel.firmwareStatus( { started: false } );
            if ( source !== "base" || !forceCheck && !ipCheck ){
                yiiModel.updateCheck( source, forceCheck );
            } else {
                var sub = yiiModel.ipChecked.subscribe( function( newValue ) {
                    if ( newValue.executed ){
                        if ( newValue.success ) {
                            yiiModel.updateCheck( source, forceCheck );
                        }
                        sub.dispose();
                    }
                });
                if ( !yiiModel.ipChecked().executed ){
                    yiiModel.checkIp();
                } else {
                    if ( yiiModel.ipChecked().success ){
                        // trigger the update of the observable again
                        yiiModel.ipChecked( { 
                            executed: true,
                            success: true 
                        });
                    }
                }
            }
        };
        yiiModel.firmwareMsg = function( data ){
            var version = data.version;
            var build = data.build;
            if ( version && !build ){
                var sp = version.split("-");
                if ( sp.length == 2 ){
                    version = sp[0];
                    build = sp[1];
                }
            }
            var msg = {
                'firmware': "Firmware version " + version + " ( build "+build+" )",
                'updater': "Updater version " + version + " ( build "+build+" )",
                'minor components': "Minor components",
                'minor': "Minor components"
            }; 
            return msg[data.type];
        };
        yiiModel.updateStatus = function () {    
            var options = {
                include_log: true
            };
            spxapi.rpc.call("firmware_update_status", [ yiiModel._firmwareUpdateConfig.handle, options ] ).done( function( data ) {
                var logs = yiiModel.firmwareStatus().logs;
                if ( !data.error && data.result.done && data.result.exit != 'ok' ){
                    if ( data.result.exit == 'fatal' ){
                        data.error = "Firmware is corrupted" ;
                    } else {
                        data.error = "Please check the logs and try again";
                    }
                    logs = data.result.log.split("\n");
                }
                if ( !data.error ){                    
                    var status = data.result;
                    if ( status.handle ) {
                        yiiModel._firmwareUpdateConfig.handle = status.handle;
                    }
                    status.started = true;
                    if ( status.log ){
                        status.logs = status.log.split("\n");
                    }
                    status.running = !status.done;
                    yiiModel.firmwareStatus( status );
                    if ( !status.done ) {
                        yiiModel._firmwareUpdateConfig.idx++;
                        setTimeout( yiiModel.updateStatus ,30*1000 );
                        // could update the notification dialog
                    } else {
                        clearFirmwareInfo( firmwareUpdateStartedId );
                        firmwareUpdateStartedId = null;   
                                                
                        var body = "<p>" + "Installed " + yiiModel.firmwareMsg(status) + "</p>";                        
                        if (!status.complete){
                            body += "<p>"+"Other updates are still awaiting installation, another update will be perfomed in order to have an up to date firmware."+"</p>";
                            var startAgain = yiiModel.reboot.subscribe(function( newVal ){
                                if ( !newVal ){
                                    yiiModel.updateStart();
                                    startAgain.dispose();
                                }
                            });
                        }
                        showFirmwareInfo( { 
                             type: 'info', 
                             title: "Firmware Updated",
                             sticky: true,
                             body: body                             
                        } );                     
                        yiiModel.reboot( true );                        
                    }
                } else {
                    yiiModel.firmwareStatus( {
                        started: false,
                        success: false,
                        reason: data.error,
                        logs: logs
                    } );                    
                }
            });
        };
        
        yiiModel.updateStart = function () {  
            firmwareUpdateStartedId = showFirmwareInfo( { 
                type: 'info', 
                title: "Firmware Update Started",
                sticky: true,
                text: "This operation might take some time. The player will automatically restart at the end of the update."
            } );
            yiiModel.firmwareInfo( { executed: false } );            
            yiiModel.firmwareStatus( { started: true, running: true } );
            var options = { 
                max_wait: 5
            };
            if ( yiiModel.firmwareSource() == 'base' || yiiModel.firmwareSource() == 'local-usb' ){
                options.repo_id = [ yiiModel.firmwareSource() ];
            } else {
                options.repo_id = [ "custom" ];
                options.repo_uri = [ { id: "custom", "uri": yiiModel.firmwareSource() }];                
            }
            
            spxapi.rpc.call("firmware_update", [ options ] ).done( function( data ) {                
                if ( !data.error ){                         
                    yiiModel._firmwareUpdateConfig = { handle: data.result, idx: 0 };                     
                    setTimeout( yiiModel.updateStatus ,10*1000 );
                } else {
                    console.log( data );
                    yiiModel.firmwareStatus( {
                        started: false,
                        success: false,
                        reason: data.error
                    } );                    
                }
            });
        };
        
        if ( config.handle ){
            firmwareUpdateStartedId = showFirmwareInfo( { 
                type: 'info', 
                title: "Firmware Update in progress",
                sticky: true,
                text: "This operation might take some time. The player will automatically restart at the end of the update."
            } );
            yiiModel.firmwareStatus( { started: true, running: true } );
            yiiModel._firmwareUpdateConfig = { handle: config.handle===true ? "" : config.handle, idx: 0 };            
        }
    }
}