<?php

/**
 * UserIdentity represents the data needed to identity a user.
 * It contains the authentication method that checks if the provided
 * data can identity the user.
 */
class UserIdentity extends CUserIdentity
{
	private $_id;
 
	/**
	 * Authenticates a user.
	 * @return boolean whether authentication succeeds.
	 */
	public function authenticate()
	{        
        $user = User::model()->findByAttributes(array('username'=>$this->username));
        if( $user === null ){
            $this->errorCode=self::ERROR_USERNAME_INVALID;
        } else if( !User::verify($this->username, $this->password, $user->password ) ){
			$this->errorCode=self::ERROR_PASSWORD_INVALID;
        }else {
			$this->_id=$user->id;
			$this->username = $user->username;
			$this->errorCode = self::ERROR_NONE;
		}
		return !$this->errorCode;
	}
    static function newFromAPIKey( $apikey )
	{
        $user = User::model()->findByAttributes(array('apikey'=>$apikey));
        if( $user === null ){
            $identity = new UserIdentity( "", "");
            $identity->errorCode=self::ERROR_UNKNOWN_IDENTITY;
        } else {
            $identity = new UserIdentity( $user->username, "" );
            $identity->_id=$user->id;			
			$identity->errorCode = self::ERROR_NONE;
        }
        return $identity;
    }
    /**
    * @return integer the ID of the user record
    */
	public function getId()
	{
		return $this->_id;
	}
}