<?php

// uncomment the following to define a path alias
// Yii::setPathOfAlias('local','path/to/local-folder');

$models = "media|playlist|playout|template|apps|external";
// This is the main Web application configuration. Any writable
// CWebApplication properties can be configured here.
$config = array(
	'basePath'=>dirname(__FILE__).DIRECTORY_SEPARATOR.'..',
    'runtimePath' => '/var/tmp/',
	'name'=>'SpinetiX',
    
    'defaultController' => 'main',

	// preloading 'log' component
	'preload'=>array(
        'log'
    ),
    'aliases' => array(
        'bootstrap' => 'application.ext.bootstrap',
        'controlcenter' => 'application.modules.controlcenter',
        'api' => 'application.modules.api',
    ),
	// autoloading model and component classes
	'import'=>array(
		'application.models.*',
		'application.components.*',
        'application.utils.*',
        'controlcenter.models.*',
        'controlcenter.controllers.*',
        'controlcenter.components.*',
        'api.models.*',
        'api.components.*',
        'bootstrap.behaviors.*',
        'bootstrap.helpers.*',
        'bootstrap.widgets.*'
	),

	'modules'=>array(
		'api' => array(
            'defaultController' => 'media',
        ),
        'assets'=> array(
            'defaultController' => 'media',
        ),
        'controlcenter'=> array(
            'defaultController' => 'main',
        ),		
	),
    
	// application components
	'components'=>array(
        'session'=>array(
		 	'sessionName'=>'uisess'
		),
		'urlManager'=>array(
			'urlFormat'=>'path',
            'showScriptName' =>false, 
			'rules'=>array(
                array('api/api/option', 'pattern'=>'api/<detail:.*>', 'verb'=>'OPTIONS'),
                
                array('api/api/list', 'pattern'=>"api/<model:(resource|schedule|$models)>/", 'verb'=>'GET'),
                array('api/api/set', 'pattern'=>"api/<model:(schedule)>/", 'verb'=>'PUT'),
                array('api/api/create', 'pattern'=>"api/<model:($models)>/", 'verb'=>'POST'),
                
                
                array('api/api/get', 'pattern'=>"api/<model:($models)>/<id:\d+>", 'verb'=>'GET'),
                array('api/api/update', 'pattern'=>"api/<model:($models)>/<id:\d+>", 'verb'=>'PUT'),
                array('api/api/delete', 'pattern'=>"api/<model:($models)>/<id:\d+>", 'verb'=>'DELETE'),
                
                array('api/api/update', 'pattern'=>'api/<model:(media)>/<id:\d+>/<detail:\w+>', 'verb'=>'PUT'),
                
                array('api/api/post', 'pattern'=>'api/<model:(media)>/<id:\d+>/', 'verb'=>'POST'),
                
                array('api/apps/<action>/', 'pattern'=>'api/apps/<id:\d+>/<action:\w+>', 'verb'=>'GET'),
                
                '<module:\w+>/<controller:\w+>/<action:\w+>'=>'<module>/<controller>/<action>',
                'assets/<controller:\w+>/<action:\w+>/<id:\w+>'=>'assets/<controller>/<action>'
                
			),
		),
		
		'db'=>array(
			'connectionString' => 'sqlite:/var/lib/spxmanage/ui/ui.db',
            'initSQLs' => array( 'PRAGMA synchronous = NORMAL;' )
		),
        'dbassets'=>array(
			'connectionString' => 'sqlite:/srv/raperca/interface/protected/assets.db',
            'class'            => 'CDbConnection',
            'initSQLs' => array( 'PRAGMA synchronous = NORMAL;' )
		),
        
        
        'authManager'=>array( 
            'class'=>'CDbAuthManager',
            'defaultRoles'=>array('authenticated', 'guest'),
        ),
		'user'=>array(
			'class'=>'application.components.WebUser', // using custom WebUser class
            'allowAutoLogin'=>true,
            'loginUrl' => array('/user/login'),
            'loginRequiredAjaxResponse' => json_encode( array(
                'success' => false,
                'errors' => array( 'Credential missing' ),
                'redirect' => '/',
            ) )
		),
/*        'errorHandler'=>array(
			// use 'site/error' action to display errors
			'errorAction'=>'error/main',
		),
 */
		'log'=>array(
			'class'=>'CLogRouter',
			'routes'=>array(
				array(
					'class'=>'MySysLogRoute',
					'levels'=>'error, warning',
                    'identity'=> 'spxmanage.yii',
                    'facility'=> LOG_USER
				)                
			),
		),
        'bootstrap' => array(
            'class' => 'bootstrap.components.BsApi'
        )
	),

	// application-level parameters that can be accessed
	// using Yii::app()->params['paramName']
	'params'=>array(
		// this is used in contact page
		'adminEmail'=>'support@spinetix.com',
	),    
    'onBeginRequest'=>array('FirmwareUpdate', 'updateInProgress'),
);
if ( defined('YII_DEBUG') ){
    $config['components']['log']['routes'][] = array(
        'class'=>'CFileLogRoute',
        'levels'=>'debug, info, error, warning',
        'logPath'=> '/var/tmp/',
        'logFile' => 'spx.log'
    );
}
return $config;