<?php

/**
 * This is the model class for table "Users".
 *
 * The followings are the available columns in table 'Users':
 * @property integer $id
 * @property string $username
 * @property string $password
 */
class User extends CActiveRecord
{
    public $verifyPassword;
    public $oldPassword = null;
    public $crypted = false;
    public $userRights = null;
    
    var $passfile; # referred in pro files directly!
    var $groupfile; 
    protected $pass_conf_name="/etc/raperca/spxpasswd.xml";    
    
    static $digest="Enter username and password";
    
    protected $users = null;
    protected $currentUser;
    protected $adminModified = false;

    public function init() {
        parent::init();
        if ( file_exists("/etc/spxmanage") ) {
            $this->passfile = "/etc/spxmanage/htpasswd";
            $this->groupfile = "/etc/spxmanage/htgroup";
        } else {
            $this->passfile = "/etc/raperca/htpasswd";
            $this->groupfile = "/etc/raperca/htgroup";
        }
    }

	/**
	 * @return string the associated database table name
	 */
	public function tableName()
	{
		return 'Users';
	}

	/**
	 * @return array validation rules for model attributes.
	 */
	public function rules()
	{
		// NOTE: you should only define rules for those attributes that
		// will receive user inputs.
		return array(
			array('username, password', 'required'),
            array('username', 'unique'),
            array('verifyPassword', 
                    'compare', 'compareAttribute'=>'password',
                    'except' => 'skipPassword'
            ),
            array('oldPassword', 'checkOldPass', 
                    'except' => 'skipPassword'
            ),
			array('username, password', 'length', 'max'=>64),
			// The following rule is used by search().
			// @todo Please remove those attributes that should not be searched.
			array('id, username', 'safe', 'on'=>'search'),
            array('apikey', 'default', 'value'=> strtoupper(md5(rand())), 'setOnEmpty'=>true ),
		);
	}
    
	/**
	 * @return array relational rules.
	 */
	public function relations()
	{
		// NOTE: you may need to adjust the relation name and the related
		// class name for the relations automatically generated below.
		return array(
		);
	}

	/**
	 * @return array customized attribute labels (name=>label)
	 */
	public function attributeLabels()
	{
		return array(
			'id' => 'ID',
			'username' => 'Username',
            'oldPassword' => 'Old Password',
			'password' => 'New Password',
            'verifyPassword'=> "Retype new Password",
            'userRights'=> "Rights",
            'apikey'=> "API Key",
		);
	}
    public function checkOldPass( $attribute ){
        
        $oldPass = $this->$attribute;
        if ( $oldPass === null ){
            return;
        }
        $currentUser = $this->getUser( $this->username );
        if ( !$this->verify($this->username, $oldPass, $currentUser['pass'])){
			$this->addError($attribute, "Old password do not match" );	
		}
    }
    
    public function setAttributes( $values, $safeOnly=true ){
        if ( isset($values['password']) && empty($values['password']) ){
            unset($values['password']);
        }
        if ( isset($values['verifyPassword']) && empty($values['verifyPassword']) ){
            unset($values['verifyPassword']);
        }
       
        if ( isset( $values['username'] ) && !empty($values['username']) && $this->currentUser && $this->currentUser['username'] != $values['username'] ){            
            $this->password = null;
        }
        if ( !isset($values['verifyPassword']) && !isset($values['password']) ){
            $this->scenario = 'skipPassword';
            $this->crypted = true;
        }
        parent::setAttributes( $values, $safeOnly );
    }
    protected function afterFind()
    {
        $this->userRights = new UserRights;
        $this->userRights->id = $this->id;
        
        $this->currentUser = $this->getUser( $this->username );
        parent::afterFind();
    }
    protected function afterSave()
    {
        if ( !$this->userRights ){
            $this->userRights = new UserRights;
            $this->userRights->id = $this->id;
        }
        parent::afterSave();
    }
    protected function afterDelete()
    {
        $this->userRights = null;
        parent::afterDelete();
    }
    protected function afterValidate()
    {
        if ( !$this->hasErrors() ){
            if ( !$this->crypted ) {                
                $this->password = self::encrypt( $this->username, $this->password );                
                $this->verifyPassword = "";
            }
            $this->crypted = true;
        }
        parent::afterValidate();
    }
    public function updateHttpdSecurity() {

        if ( $this->currentUser && $this->scenario == 'skipPassword' ){
            $password = $this->currentUser['pass'];            
        } else {
            $password = $this->password;
        }
        if ( $this->userRights === null || ( $this->currentUser && $this->currentUser['username'] != $this->username ) ) {
            $this->clearUser( $this->currentUser['username'] );
        }        
        if ( $this->userRights !== null ){
            
            $this->addUser( $this->username, $password, $this->userRights->getUserRights(), true);
        }
        
        return $this->saveUsers();
    }
    
    static function encrypt( $username, $password ){ 
        $info = new BasicInfo();
        $fw = explode( ".", $info->getFirmware() );
        if ( $fw[0] > 3 && defined("CRYPT_BLOWFISH") && CRYPT_BLOWFISH ){
            $salt = '$2y$07$' . self::getRandomString(22) . '$';
        } else {
            $salt = '$1$' . self::getRandomString(8) . '$';
        }
        return crypt($password, $salt);
    }
    static function verify( $username, $password, $hash ){         
        return $hash == crypt($password, $hash);
    }
	/**
	 * Retrieves a list of models based on the current search/filter conditions.
	 *
	 * Typical usecase:
	 * - Initialize the model fields with values from filter form.
	 * - Execute this method to get CActiveDataProvider instance which will filter
	 * models according to data in model fields.
	 * - Pass data provider to CGridView, CListView or any similar widget.
	 *
	 * @return CActiveDataProvider the data provider that can return the models
	 * based on the search/filter conditions.
	 */
	public function search()
	{
		// @todo Please modify the following code to remove attributes that should not be searched.

		$criteria=new CDbCriteria;

		$criteria->compare('id',$this->id);
		$criteria->compare('username',$this->username,true);

		return new CActiveDataProvider($this, array(
			'criteria'=>$criteria,
		));
	}

	/**
	 * Returns the static model of the specified AR class.
	 * Please note that you should have this exact method in all your CActiveRecord descendants!
	 * @param string $className active record class name.
	 * @return User the static model class
	 */
	public static function model($className=__CLASS__)
	{
		return parent::model($className);
	}
    
    function clearAllUsers() {
        // remove all users except admin        
        $users = self::model()->findAll();
        foreach ( $users as $user ){
            if ( $user->id > 1 ){
                $this->clearUser( $user->username );
                $user->delete();
            }            
        }
        return $this->saveUsers();        
    }
    function getUsers( ){
        
        $this->users=array();
        
        if ( file_exists($this->passfile) ){
            $groups = array();
            if ( file_exists($this->groupfile) ){
                $groupFile = file_get_contents($this->groupfile);
            } else {
                $groupFile = "";
            }
            $groupsStr = explode("\n",$groupFile);
            foreach ( $groupsStr as $groupStr) {
                $data = explode( ": ", $groupStr );
                if ( count($data) == 2 ){
                    $groups[$data[0]] = explode( " ", $data[1] );
                }
            }
            if ( file_exists($this->passfile) ){
                $passwordsFile = file_get_contents($this->passfile);
            } else {
                $passwordsFile = "";
            }
            $passwords = explode("\n",$passwordsFile);
            $passelements = array();
            
            foreach ($passwords as $line){
                if (preg_match("/(.*):(.*)/", $line, $passelements)){
                    $user = array(
                        'username'=>$passelements[1], 
                        'pass'=>$passelements[2], 
                        'groups' => array()
                    );
                    foreach ( $groups as $group => $items ){
                        if (in_array( $user['username'], $items) ){
                            $user['groups'][$group] = 1;
                        } else {
                            $user['groups'][$group] = 0;
                        }
                    }
                    $this->users[] = $user;
                }            
            }
            
        }
        
        return $this->users;
    }
    
    static function getRandomString( $length ) {
        $res = "";
      
        if ( PHP_OS == "WINNT" ) {
            $pr_bits = "";
            for ($i=0; $i<$length; $i++)
                $pr_bits .= chr(mt_rand(0,255)); // this is not good, but just for testing on Windows OS
        } else {
            $fp = fopen('/dev/urandom','rb');
            if ($fp !== FALSE) {
                $pr_bits = fread($fp,$length);
                fclose($fp);
            }
        }
        $res = strtr(base64_encode($pr_bits), array("/"=>"", "+"=>"") );
        return substr( $res, 0, $length );
        
    }
    
    function saveUsers( ){
        if ( $this->users === null ){
            $this->getUsers();
        }                    
        $xlinkNS = 'http://www.w3.org/1999/xlink';
        
        $pass_config = new DOMDocument;
        if ( file_exists($this->pass_conf_name) ){
            $pass_config->loadXML(file_get_contents($this->pass_conf_name));
        }
        $parent = $pass_config->getElementsByTagName('credits');
        if ($parent->length<1) {
            $parent = $pass_config->appendChild($pass_config->createElement('credits'));
        } else {
            $parent=$parent->item(0);
        }
        
        $str="";
        $groupStr = array();
        
        // add a random password for the uploader to be able to access the content, the logs and the interface
        $username = 'uploader';
        $pass = "";
        $cred = $pass_config->getElementById("admin");
        if ( $cred ) {
            $pass = $cred->getAttribute("password");
        }
        if ( $pass == "" ){
            $pass = self::getRandomString(16);
        }
        $this->addUser( $username, $pass, array( 'admin' => true ) );
        
        foreach( $this->users as $user ){
            $str .= $user['username'].":".$user['pass']."\n";            
            if ( isset($user['groups']) ){
                foreach ( $user['groups'] as $group => $active ){
                    if ( $active ) {
                        if ( !isset( $groupStr[$group] )){
                            $groupStr[$group] = "$group:";
                        }
                        $groupStr[$group] .= " ".$user['username'];
                    }
                }
            }
        }
        
        if ( empty($str) ){
            if ( file_exists( $this->passfile) )
                unlink( $this->passfile );
            if ( file_exists( $this->groupfile) )
                unlink( $this->groupfile );
            return true;
        } else {
            $ret = MaintenanceJobs::saveFile( $this->passfile, $str );
            if ( $ret=== true ){
                if ( !MaintenanceJobs::saveFile( $this->groupfile, implode("\n", $groupStr ) ) ) {
                    $ret = false;
                    $this->addError( 'password', "Failed setting password groups, please set again." );
                }
            } else {
                $this->addError( 'password', "Failed setting password, please set again." );
            }
            // save data in the XML file also        
            if ( $ret===true ) {    
                $id_set = array(
                    "admin"=>'http://localhost/', 
                    "content"=>'http://localhost:81/', 
                    "admin2"=>'http://127.0.0.1/', 
                    "content2"=>'http://127.0.0.1:81/'
                );
                foreach ($id_set as $id=>$dest ) {
                    $cred = $pass_config->getElementById($id);
                    if ( !$cred ) {
                        $cred=$pass_config->createElement('creditentials');
                        $parent->appendChild($cred);
                    } 
                    $cred->setAttribute("username", $username);
                    $cred->setAttribute("password", $pass);
                    $cred->setAttributeNS($xlinkNS,"xlink:href", $dest);
                    $cred->setAttribute("xml:id", $id);
                } 
                if ( !MaintenanceJobs::saveFile($this->pass_conf_name, $pass_config->saveXML(), true) ) {
                    $ret = false;
                    $this->addError( 'password', "Failed setting password credentials, please set again." );
                }      
            }
            if ( $ret===true && $this->adminModified ) {
                // ideally should create a smaller file with only the admin user.
                if ( !MaintenanceJobs::setSafeModeData("htpasswd", $this->passfile ) ){
                    $this->addError( 'password', "Failed setting password for recovery console, please set again." );
                    return false;
                }
            }
            return $ret;
        }
        
    }
    function clearUser( $name ) {
        if ( $this->users === null ){
            $this->getUsers();
        }
        foreach ( $this->users as $idx => $user ){
            if ( $user['username'] == $name ){
                unset( $this->users[$idx] );
            }
        }
        return $this->users;
    }
    function getUser( $name, &$idx=0 ){
        if ( $this->users === null ){
            $this->getUsers();
        }
        $currentUser = false;
        foreach( $this->users as $idx => $user){
            if ( $user['username'] == $name ) {
                $currentUser = $user;
                break;
            }
        }
        return $currentUser;
    }
    function addUser( $username, $newpass, $groups = null, $encripted=false){
        if ( $this->users === null ){
            $this->getUsers();
        }
        $idx=-1;
	    $currentUser = $this->getUser($username, $idx);

        if (!$currentUser){
            $currentUser = array();
            $currentUser['username'] = $username;            
            $idx=-1;
        } 
        if ( $groups ){                
            $currentUser['groups'] = $groups;
        }
        if ( $newpass ) {
            if ( $encripted ) {
                $currentUser['pass'] = $newpass;
            } else {
                $currentUser['clearpass']=$newpass;
                $currentUser['pass'] = self::encrypt($currentUser['username'], $newpass);                
            }
        }
        if ($idx>=0)
            $this->users[$idx] = $currentUser;
        else
            $this->users[] = $currentUser;
        if ( $currentUser['username'] == 'admin' ){
            $this->adminModified = true;
        }     
        return $this->users;    
    }
        
    function crypt_apr1_md5($plainpasswd, $salt="") {
        if ($salt=="")
            $salt = substr(str_shuffle("abcdefghijklmnopqrstuvwxyz0123456789"), 0, 8);
        else 
            $salt = substr($salt,6,8);
     
        $len = strlen($plainpasswd);
        $text = $plainpasswd.'$apr1$'.$salt;
        $bin = pack("H32", md5($plainpasswd.$salt.$plainpasswd));
        for($i = $len; $i > 0; $i -= 16) { $text .= substr($bin, 0, min(16, $i)); }
        for($i = $len; $i > 0; $i >>= 1) { $text .= ($i & 1) ? chr(0) : $plainpasswd{0}; }
        $bin = pack("H32", md5($text));
        for($i = 0; $i < 1000; $i++) {
            $new = ($i & 1) ? $plainpasswd : $bin;
            if ($i % 3) $new .= $salt;
            if ($i % 7) $new .= $plainpasswd;
            $new .= ($i & 1) ? $bin : $plainpasswd;
            $bin = pack("H32", md5($new));
        }
        $tmp="";
        for ($i = 0; $i < 5; $i++) {
            $k = $i + 6;
            $j = $i + 12;
            if ($j == 16) $j = 5;
            $tmp = $bin[$i].$bin[$k].$bin[$j].$tmp;
        }
        $tmp = chr(0).chr(0).$bin[11].$tmp;
        $tmp = strtr(strrev(substr(base64_encode($tmp), 2)),
        "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/",
        "./0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz");
        return "$"."apr1"."$".$salt."$".$tmp;
    }
}
