<?php
class ControlCenterController extends Controller
{
	public $response = array();
    protected $uniqueId = 0;
    
    public $pages;

    public function accessRules()
    {
        $rules = array(      
            array('allow',                
                'roles'=>array('useControlCenter'),
            ),
            array('deny',
                'users'=>array('*')
            )   
        );
        
        return $rules;
    }
    
    function errorSummary( $model ) {
        Yii::app()->clientScript ->registerScript( 
            'yiiErrors', "var yiiErrors = [];", CClientScript::POS_BEGIN
        );
        $script = "";
        foreach ( $model->getErrors() as $errors ){
            foreach ( $errors as $error) {
                $script .= "yiiErrors[] = ".CJavaScript::encode($error).";\n";
            }
        }
        echo CHtml::script( $script )."\n";
    }
  
    
    public function renderHybrid( $view, $options ){
        if ( Yii::app()->request->isAjaxRequest ){
            $this->layout = '//layouts/jason';
            $data = null;
            $messages = null;
            $errors = array();
            if( isset( $options['data'] ) ){
                $data = $options['data'];
                unset( $options['data'] );
            }
            if( isset( $options['messages'] ) ){
                $messages = $options['messages'];
                unset( $options['messages'] );
            }
            foreach( $options as $obj ){
                if ( !is_object($obj) )
                    continue;
                $errors = $this->updateErrors($obj, $errors);
            }
            $this->render('/layouts/ajax', array(
                'errors' => $errors,
                'messages' => $messages,
                'data' => $data
            ));   
        } else {
            $this->render( $view, $options );
        }                
    }
    public function updateErrors( $model, $errors = array() ){         
        foreach ( $model->getErrors() as $name => $error ){
            $previousErrors = array();
            foreach ( $error as $e ){
                if ( !in_array($e, $previousErrors) ){
                    if ( !empty($name) ){
                        $errors[] = $name .": ".$e;
                    } else {
                        $errors[] = $e;
                    }
                    $previousErrors[] = $e;
                }
            }
        }
        return $errors;
    }
    public function init(){
        parent::init();
        
        $this->layout='/layouts/base';
        $applicationsVisible = Yii::app()->branding->hasRight('interactivity') |
                Yii::app()->branding->hasRight('interactivity') |
                Yii::app()->branding->hasRight('netAPI') |
                Yii::app()->branding->hasRight('multiscreen') |
                Yii::app()->branding->hasRight('NTP') |
                Yii::app()->branding->hasRight('streaming') |
                Yii::app()->branding->hasRight('firmware') |                
                Yii::app()->branding->hasRight('pullMode');
        if (  Yii::app()->user->checkAccess('useControlCenter') ){
            $this->pages = array( 
                array( 'label' => 'Control Center', 'url'  => array( '/controlcenter/main/index' )
                ),            
                array( 'label' => 'System', 'url'  =>  array( '/controlcenter/main/system' ), 'items' => array(
                    array( 'label' => 'Configuration Backup', 'url'  => array( '/controlcenter/main/system', '#'=>'Backup' ) ),
                    array( 'label' => 'Identity', 'url'  => array( '/controlcenter/main/system', '#'=>'Identity' ) ),
                    array( 'label' => 'Time Zone', 'url'  => array( '/controlcenter/main/system', '#'=>'TimeZone' ) ),
                    array( 'label' => 'Date & Time', 'url'  => array( '/controlcenter/main/system', '#'=>'DateTime' ) ),  
                    array( 'label' => 'Hardware Settings', 'url'  => array( '/controlcenter/main/system', '#'=>'Hardware' ) ),
                    array( 'label' => 'Saved Passwords', 'url'  => array( '/controlcenter/main/system', '#'=>'SavedPasswords' ) ),                
                    array( 'label' => 'Cockpit', 'url' => array( '/controlcenter/main/system', '#'=>'Cockpit' ) ),
                ) ),            
                array( 'label' => 'Display & Audio', 'url'  => array( '/controlcenter/main/display' ), 'items' => array(
                    array( 'label' => 'Display', 'url'  => array( '/controlcenter/main/display', '#'=>'Display' ) ),
                    array( 'label' => 'Audio', 'url'  => array( '/controlcenter/main/display', '#'=>'Audio' ) ),
                    array( 'label' => 'Power', 'url'  => array( '/controlcenter/main/display', '#'=>'Power' ) ),
                    array( 'label' => 'Other', 'url'  => array( '/controlcenter/main/display', '#'=>'Other' ) ),
                ) ),
                array( 'label' => 'Network', 'url'  => array( '/controlcenter/main/network' ), 'items' => array(    
                    array( 'label' => 'IP Configuration', 'url'  => array( '/controlcenter/main/network', '#'=>'IP' ) ),
                    array( 'label' => 'HTTP Proxy', 'url'  => array( '/controlcenter/main/network', '#'=>'Proxy' ) ),
                    array( 'label' => 'HTTPS', 'url'  => array( '/controlcenter/main/network', '#'=>'HTTPS' ) ),
                    array( 'label' => 'Watchdog', 'url'  => array( '/controlcenter/main/network', '#'=>'Watchdog' ),
                        'visible' => Yii::app()->branding->hasRight('networkWatchdog') ),
                    array( 'label' => 'SNMP', 'url'  => array( '/controlcenter/main/network', '#'=>'SNMP' ),
                        'visible' => Yii::app()->branding->hasRight('networkSNMP') ),
                ) ),
                array( 'label' => 'Content', 'url'  => array( '/controlcenter/main/content' ), 'items' => array(
                    array( 'label' => 'Scheduled Download', 'url'  => array( '/controlcenter/main/content', '#'=>'Download' ),
                        'visible' => Yii::app()->branding->hasRight('pullMode') ),
                    array( 'label' => 'Live Sources', 'url'  => array( '/controlcenter/main/content', '#'=>'Live' ),
                        'visible' => Yii::app()->branding->hasRight('contentLiveSource') ),        
                    array( 'label' => 'Splash Screens', 'url'  => array( '/controlcenter/main/content', '#'=>'Splash' ) ),                
                ) ),
                array( 'label' => 'Peripherals', 'url'  => array( '/controlcenter/main/peripherals' ), 'items' => array(
                    array( 'label' => 'Serial', 'url'  => array( '/controlcenter/main/peripherals', '#'=>'Serial' ),
                        'visible' => Yii::app()->branding->hasRight('peripheralsSerial') ),
                    array( 'label' => 'Automation', 'url'  => array( '/controlcenter/main/peripherals', '#'=>'FSM' ),
                        'visible' => Yii::app()->branding->hasRight('peripheralsFSM') ),
                    array( 'label' => 'USB', 'url'  => array( '/controlcenter/main/peripherals', '#'=>'USB' ),
                        'visible' => Yii::app()->branding->hasRight('peripheralsUSB') ),                
                ) ), 
                array( 'label' => 'Advanced Applications', 'url'  => array( '/controlcenter/main/applications' ), 'items' => array(
                    array( 'label' => 'Interactivity', 'url'  => array( '/controlcenter/main/applications', '#'=>'Interactivity' ),
                        'visible' => Yii::app()->branding->hasRight('interactivity') ),
                    array( 'label' => 'Touchscreen', 'url'  => array( '/controlcenter/main/applications', '#'=>'Touchscreen' ),
                        'visible' => Yii::app()->branding->hasRight('interactivity') ),
                    array( 'label' => 'Network API', 'url'  => array( '/controlcenter/main/applications', '#'=>'SV' ),
                        'visible' => Yii::app()->branding->hasRight('netAPI') ),
                    array( 'label' => 'Mutltiscreen', 'url'  => array( '/controlcenter/main/applications', '#'=>'Multiscreen' ),
                        'visible' => Yii::app()->branding->hasRight('multiscreen') ),
                    array( 'label' => 'NTP', 'url'  => array( '/controlcenter/main/applications', '#'=>'NTP' ),
                        'visible' => Yii::app()->branding->hasRight('NTP') ),   
                    array( 'label' => 'Streaming', 'url'  => array( '/controlcenter/main/applications', '#'=>'Streaming' ),
                        'visible' => Yii::app()->branding->hasRight('streaming') ),   
                    array( 'label' => 'Firmware', 'url'  => array( '/controlcenter/main/applications', '#'=>'Firmware' ),
                        'visible' => Yii::app()->branding->hasRight('firmware') ),   
                    array( 'label' => 'RPC Concentrator', 'url'  => array( '/controlcenter/main/applications', '#'=>'RPC' ),
                        'visible' => Yii::app()->branding->hasRight('pullMode') ),                   
                    array( 'label' => 'Pull Mode', 'url'  => array( '/controlcenter/main/applications', '#'=>'PullMode' ),
                        'visible' => Yii::app()->branding->hasRight('pullMode') ),                   
                ), 'visible' => $applicationsVisible),                 
                array( 'label' => 'Operations', 'url'  => array( '/controlcenter/main/operations' ), 'items' => array(
                    array( 'label' => 'Firmware Update', 'url'  => array( '/controlcenter/main/operations', '#'=>'Firmware' ) ),
                    array( 'label' => 'Maintenance', 'url'  => array( '/controlcenter/main/operations', '#'=>'Maintenance' ) ),
                    array( 'label' => 'Reset', 'url'  => array( '/controlcenter/main/operations', '#'=>'Reset' ) ),                                
                    array( 'label' => 'Support', 'url'  => array( '/controlcenter/main/operations', '#'=>'Support' ) ),                                
                ) ),
                array( 'label' => 'Logs', 'url'  => array( '/controlcenter/main/logs' ), 'items' => array(
                    array( 'label' => 'Proof of Play', 'url'  => array( '/controlcenter/main/logs', '#'=>'Proof' ) ),
                    array( 'label' => 'Current', 'url'  => array( '/controlcenter/main/logs', '#'=>'Current' ) ),
                    array( 'label' => 'Backup', 'url'  => array( '/controlcenter/main/logs', '#'=>'Backup' ) ),                                
                    array( 'label' => 'Scheduled Upload', 'url'  => array( '/controlcenter/main/logs', '#'=>'Upload' ),
                        'visible' => Yii::app()->branding->hasRight('pullMode') ),                                
                ), 'visible' => Yii::app()->branding->hasRight('log') ),
                array( 'label' => 'Advanced', 'url'  => array( '/controlcenter/main/advanced' ), 'items' => array(
                    array( 'label' => 'Player', 'url'  => array( '/controlcenter/main/advanced', '#'=>'Player' ) ),
                    array( 'label' => 'Log', 'url'  => array( '/controlcenter/main/advanced', '#'=>'Log' ) ),
                    array( 'label' => 'Appender', 'url'  => array( '/controlcenter/main/advanced', '#'=>'Appender' ) ),
                    array( 'label' => 'New Appender', 'url'  => array( '/controlcenter/main/advanced', '#'=>'NewAppender' ) ),
                    array( 'label' => 'Logger', 'url'  => array( '/controlcenter/main/advanced', '#'=>'Logger' ) ),
                ), 'visible' => Yii::app()->user->checkAccess('useAdvancedConfig') ),
                array( 'label' => 'About', 'url'  => array( '/controlcenter/main/about' )
                ), 
            );
        } else {
            $this->pages = array( 
                array( 'label' => 'Control Center', 'url'  => array( '/controlcenter/main/index' )
                ),            
                array( 'label' => 'Logs', 'url'  => array( '/controlcenter/main/logs' ), 'items' => array(
                    array( 'label' => 'Proof of Play', 'url'  => array( '/controlcenter/main/logs', '#'=>'Proof' ) ),
                    array( 'label' => 'Current', 'url'  => array( '/controlcenter/main/logs', '#'=>'Current' ) ),
                    array( 'label' => 'Backup', 'url'  => array( '/controlcenter/main/logs', '#'=>'Backup' ) ),                                                                                  
                ), 'visible' => Yii::app()->branding->hasRight('log') ),
                array( 'label' => 'About', 'url'  => array( '/controlcenter/main/about' )
                ), 
            );
        }
    }    
}