<?php

class FirmwareController extends ControlCenterController
{
    protected $debug = false;
    public function accessRules()
    {
        $rules = array(            
            array('allow',
                'actions' => array( 'check' ),
                'roles'=>array('viewLogs'),
            )
        );
        
        return array_merge( $rules, parent::accessRules() ); ;
    }
    public function status( $firmware ){
        $results = array();
        if ( $firmware->updater->updater_status===false ){
            // update not running but log is not complete
            $status = $firmware->updater->getStatus();
            if ( $status=="READY" || $status=="REBOOTING" ){
                $results['success'] = true;
            } elseif ( $status=="CORRUPTED" ) {
                $results['success'] = false;
                $results['reason'] = "Firmware is corrupted";
            } else {
                $results['success'] = false;
                $results['reason'] = "Unknown status: $status";
            }
        } else {
            if ( $firmware->updater->updater_status=="OK")
                $results['success'] = true;
            else {
                $results['success'] = false;
                $results['reason'] = "Update failed: ".$firmware->updater->updater_status;
            }
        }
        return $results;
    }
    public function actionCheck( $updatesource = 'base', $forceCheck = false )
	{
        if ( PHP_OS == "WINNT" ) {
            sleep( 2 );
        }
		if ( !Yii::app()->request->isAjaxRequest )
			throw new CHttpException(403 , "Direct access not allowed");
	
		
		if ( $this->debug ) {
			$nbUpdates = 10;
			$results = array(	'success'=>true,
								'updates'=>$nbUpdates );
			$results ['version'] = "x.x.x";
			echo json_encode( $results );
			return;
		}
        
        $firmware = new FirmwareUpdate;
        // start checking for the firmware
		$firmware->setScenario('check');
		$firmware->setAttributes( array( 
            'mode'=>'check',
            'updatesource' => $updatesource,
            'forceCheck' => $forceCheck
        ) );
		if ( $firmware->startUpdate( true ) ) {
            
            $logs_str = $firmware->getLog( );
            $nbUpdates = $firmware->getNbUpdates();
            
            $results = $this->status( $firmware );
            $results += array(
                              'updates'=> intval( $nbUpdates ),
                              'logs' => $logs_str,
            );
                              
            if ( $nbUpdates!=0 ){
                if ( $firmware->updater->update_info['type'] == 'FIRMWARE') {
                    $results['type'] = 'firmware';
                    $results['version'] = $firmware->updater->update_info['ver'];
                    $results['build'] = $firmware->updater->update_info['rel'];
                } elseif ( $firmware->updater->update_info['type'] == 'UPDATER') {
                    $results['type'] = 'updater';
                    $results['version'] = $firmware->updater->update_info['ver'];
                    $results['build'] = $firmware->updater->update_info['rel'];
                } else {
                    $results['type'] = 'minor components';
                }
                $results['source'] = $updatesource;
            }
        } else {
             $results = array(  'success'=>false,
                                'reason'=>$firmware->updater->errorStr,
                                'logs' => $firmware->getLog( ) );
        }        
		echo json_encode( $results );
	}
	
	public function actionUpdate()
	{
        if ( PHP_OS == "WINNT" ) {
            sleep( 2 );
        }
		if ( !Yii::app()->request->isAjaxRequest )
			throw new CHttpException(403 , "Direct access not allowed");
		
			
		if ( $this->debug ) {
			$log_id = rand( 0, 10 );
			$results = array( 'success'=>true,
                              'id'=>$log_id );
			echo json_encode( $results );
			return;
		}
		$mode = 'update';
        if ( isset( $_GET['mode'] ) ) {
            $mode = $_GET['mode'];
        }
        $updatesource = 'server';
        if ( isset( $_GET['updatesource'] ) ) {
            $updatesource = $_GET['updatesource'];
        }
        
        $firmware = new FirmwareUpdate;
        // start checking for the firmware
		$firmware->setScenario('update');
		$firmware->setAttributes( array( 
            'mode'=>$mode,
            'updatesource'=>$updatesource,
        ) );
		if ( $firmware->startUpdate( false ) ) {
            $log_id = $firmware->log_id;
            
            $results = array( 'success'=>true,
                              'id'=>$log_id );
        } else {
             $results = array(  'success'=>false,
                                'reason'=>$firmware->updater->errorStr,
                                'logs' => $firmware->getLog( ) 
             );
        }        
		echo json_encode( $results );
	}
	public function actionStatus()
	{
		if ( !Yii::app()->request->isAjaxRequest )
			throw new CHttpException(403 , "Direct access not allowed");
		
		if ( !isset( $_GET['id'] ) ) {
			$results = array( 'success'=>false,
                              'reason'=>'Missing id' );
			echo json_encode( $results );
			return;
		}
		
		if ( $this->debug ) {
			if ( intval($_GET['idx'])<2 ){
				$results = array( 
                    'success'=>true, 
                    'running'=>true,
                    'id' => $_GET['id']
                );
			} else {
				$results = array( 'running'=>false );
				$results['success'] = true;
				$results['type'] = 'firmware';
				$results['version'] = "x.x.x";
				$results['build'] = "xxxxx";
                $results['id'] = $_GET['id'];
			}
			echo json_encode( $results );
			return;
		}
		
        $firmware = new FirmwareUpdate;
        // start checking for the firmware
		$firmware->setScenario('update');
		$firmware->setAttributes( array( 'mode'=>'update', 'log_id'=>$_GET['id'] )  );
		$running = $firmware->isUpdateRunning();
		$logs_str = $firmware->getLog( );
		
		if ( $running ) {
			$results = array( 
                'success'=>true, 
                'running'=>true,
                'id' => $_GET['id']
            );
		} else {
			$results = array( 
                'running'=>false,
                'id' => $_GET['id']
            );
            $results += $this->status( $firmware );            
			if ( $results['success'] ) {
				if ( $firmware->updater->update_info!==false ){
					if ( $firmware->updater->update_info['type'] == 'FIRMWARE') {
						$results['type'] = 'firmware';
						$results['version'] = $firmware->updater->update_info['ver'];
						$results['build'] = $firmware->updater->update_info['rel'];
					} elseif ( $firmware->updater->update_info['type'] == 'UPDATER') {
						$results['type'] = 'updater';
						$results['version'] = $firmware->updater->update_info['ver'];
						$results['build'] = $firmware->updater->update_info['rel'];
					} else {
						$results['type'] = 'minor components';
					}
				} else {
					$results['type'] = 'minor components';
				}
				$results['more'] = $firmware->updater->updater_fallback;	
			} 
		}
        $results['logs'] = $logs_str;
		echo json_encode( $results );
	}
}