<?php
/* @var $this MainController */

$this->registerSpxScript( 'firmware.js' );
$configFirmware = array(
    'urlNetCheck' => CHtml::normalizeUrl(array("operation/checkNetwork")),
    'urlFirmware' => CHtml::normalizeUrl(array("firmware/check"))    
);
$summary = CHtml::normalizeUrl(array("main/summary"));

if ( Yii::app()->user->checkAccess('useControlCenter') ){
    $tempLink = "You can increase the shutdown temperature using the ". CHtml::link("configuration page", array('system', "#"=>"Hardware") ) .".";
    $maintenanceLink = CHtml::link("Go to Maintenance...", array( 'operations' ,'#'=> 'Maintenance' ) );
    $ipConfigLink = CHtml::link("Go to IP Configuration...", array( 'network' ,'#'=> 'IP' ) );
    $displayLink = CHtml::link("Go to Display...", array( 'display' ,'#'=> 'Display' ) );
    $displayPowerLink = CHtml::link("Go to Display power management...", array( 'display' ,'#'=> 'Power' ) );
    $storageLink = CHtml::link("Manage Content...", array( 'content' ,'#'=> 'Download' ) ) ."<br/>" .
                   CHtml::link("Reset Storage...", array( 'operations' ,'#'=> 'Reset' ) );
    $logLink = CHtml::link("Show logs...", array( 'logs' ,'#'=> 'Current' ) );
} else {
    $tempLink = "";
    $maintenanceLink = CHtml::link("Show logs...", array( 'logs' ,'#'=> 'Current' ) ); 
    $ipConfigLink = "";
    $displayLink = "";
    $storageLink = "";
    $logLink = CHtml::link("Show logs...", array( 'logs' ,'#'=> 'Current' ) );    
}

if (Yii::app()->user->info->model !== 'Bonsai') {
    if ( $basicInfo->getTemp_max() || $basicInfo->getTemp_crit() ) {
        $this->widget("Gritter", array(
            'title' => "Maximum temperature reached",
            'desc' => "The player will shutdown if the condition persists." ." ".
            $tempLink,
        ));
    }
}

ob_start(); 
$table =  $this->widget('DataTable', array( 'data' => array(
        array( 'label'=>"Name", 'value' => $playerConfig->deviceName ),
        array( 'label'=>"Time zone" , 'value' => $timeSettings->getTimeZoneString() ),
        array( 'label'=>"Display settings" , 'value' => $screenInfo->settingsStr )        
    ) ), true );
$this->beginWidget('bootstrap.widgets.BsPanel', array(
    'title' => 'Current Settings',
    'beforeBody' => $table
));

    if ( Yii::app()->user->checkAccess('useControlCenter') ){
        $this->widget('BsRow', array(
            'cols' => array( 
                array( 
                    'size' => 6, 'offset' => 6,
                    'content' => BsHtml::button('Configuration Wizard', array(
                        'block' => true,                 
                        'color' => BsHtml::BUTTON_COLOR_PRIMARY, 
                        'data-bind' => "click: function() { yiiWizard.show(); }" 
                    ) )
                )
            )
        ));
    } else {
        echo $logLink;
    }
$this->endWidget();
$Settings = ob_get_contents ( );
ob_end_clean();

ob_start(); 
$this->beginWidget('bootstrap.widgets.BsPanel', array(
    'title' => 'Quick Links'
));
    $cockpitEnabled = $pullModeConfig->cockpitUri != null;
    $cockpitHost = "https://cockpit.spinetix.com";
    $cockpitUri = "/users/remote_login";
    $cockpitNewPlayer = "/users/concentrator_url.json";

    if ( $cockpitEnabled ){
        $cockpitTxt = BsHtml::linkButton('Open in Cockpit', array( 'block' => true, 'target' => '_blank', 'url' => $cockpitHost . '/players/'.$this->serial ."/search") ) ;
    } else if ( Yii::app()->user->checkAccess('useControlCenter') ){
        $cockpitTxt = 
            CHtml::beginForm(  array( ), "POST" ).
            CHtml::activeHiddenField($pullModeConfig, "cockpitUri", array( 'id' => 'CockpitUri' )) .        
            $this->widget('ActionButton', array( 
                    'label' => 'Monitor by Cockpit', 
                    'options' => array( 'id' => 'CockpitMonitor' ),
                    'block' => true, 
                    'post' => true
            ), true ) .
            CHtml::endForm() .
            BsHtml::linkButton('Configure Cockpit', array( 'id' => 'GoSystem', 'block' => true, 'url' => array( "main/system", "#" => 'Cockpit' )) );
    } else {
        $cockpitTxt = "";
    }
    if ( Yii::app()->user->checkAccess('useControlCenter') ){
        $this->widget('BsRow', array(
        'cols' => array( 
            array( 'size' => 6, 
                'content' => $this->widget('ActionButton', array( 
                    'label' => 'Restore Config', 
                    'block' => true, 
                    'link' => array( "backup/post" ), 
                    'file' => CHtml::activeName( $configBackup, 'uploadedfile' ) 
            ), true ) ),
            array( 'size' => 6, 'content' => $cockpitTxt ),
            array( 'size' => 6, 
                'content' => $this->widget('ActionButton', array( 
                    'label' => 'Generate Report', 
                    'block' => true, 
                    'link' => array("operation/report") 
            ), true ) ),
            array( 'size' => 6, 
                'content' => $this->widget('ActionButton', array( 
                    'label' => 'Restart Now', 
                    'block' => true, 
                    'link' => array("operation/reboot", 'reboot'=>true, ), 
                    'ko' => 'doAjaxButton' 
            ), true ) ),
        ) ) );
    } else {
        $this->widget('BsRow', array(
        'cols' => array( 
            array( 'size' => 6, 'content' => $cockpitTxt ),
            array( 'size' => 6, 
                'content' => $this->widget('ActionButton', array( 
                    'label' => 'Generate Report', 
                    'block' => true, 
                    'link' => array("operation/report") 
            ), true ) ),            
        ) ) );
    }
$this->endWidget();
$Links = ob_get_contents ( );
ob_end_clean();

$verticalSnapshot = SnapshotDisplay::isVertical( $playerConfig->orientation );
ob_start(); 
$this->beginWidget('bootstrap.widgets.BsPanel', array(
    'title' => 'Snapshot'
));
    echo CHtml::openTag("div", array( 
        "class"=>"content",  "style" => "min-height: 110px;"      
    )). "\n";
    
    $this->widget('SnapshotDisplay', array(
        'resolution' => $screenInfo->resolution,
        'aspect' => $screenInfo->aspect,
        'orientation' => $playerConfig->orientation,
    ) );
        
    echo CHtml::tag("div", array( "style" => "margin-top: 15px;" ) , " " );
    /*
    
    echo CHtml::tag("div", array(
        "id" => 'snapshot',
        "class" => "spx-preview spx-preview-fixed-item spx-preview-background",
        "style" => "background-image: url(/snapshot);"
    ), " ");       
    
     */
    echo CHtml::closeTag("div");      
      
    $this->widget('BsRow', array(
        'cols' => array( 
            array( 
                'size' => 6, 'offset' => 6,
                'content' => BsHtml::button('Refresh', array(
                    'block' => true,                 
                    'data-bind' => "click: function() { var d = new Date(); $('#snapshot').attr('src', '/snapshot?_=' + d.getTime() ); }"
                ) )
            )
        )
    ));
$this->endWidget();
$Snapshot = ob_get_contents ( );
ob_end_clean();

ob_start(); 
$this->beginWidget('bootstrap.widgets.BsPanel');
    if ( $verticalSnapshot ){
        $sizesImg = array( 'xs' => 6, 'sm' => 12, 'md' => 6, 'lg' => 4, 'xl' => 2);
        $sizes = array( 'xs' => 6, 'sm' => 12, 'md' => 6, 'lg' => 4, 'xl' => 2);  
    } else {
        $sizesImg = array( 'xs' => 6, 'sm' => 4, 'md' => 3, 'xl' => 2 );
        $sizes = array( 'xs' => 6, 'sm' => 4, 'md' => 3, 'xl' => 2 );        
    }
    $row = $this->beginWidget('BsRow');
        $row->beginCol( $sizesImg , array( 'style'=> 'height:150px;' ) );
            echo CHtml::openTag("div", array(  ));
            echo BsHtml::imageResponsive("/images/product.jpg", "Product", array( "style" => "margin: auto"));
            echo BsHtml::emphasis( "" , array(
                'data-bind' => "text: summaryData.time",
                'color' => BsHtml::TEXT_ALIGN_CENTER
            ));            
            echo CHtml::closeTag("div");
        $row->endCol();

        $row->addCol( array( 
            'sizes'=>$sizes, 
            'content'=> $this->widget('ButPro', array( 
                            'title' => 'UP TIME', 
                            'text' => 'js:summaryData.uptime',
                            'popover' => array( "Up Time", $maintenanceLink )
                        ), true ) 
        ) );
        $row->addCol( array( 
            'sizes'=>$sizes, 
            'content'=> $this->widget('ButPro', array( 
                            'title' => 'CPU LOAD', 
                            'text' => "js:summaryData.cpuInfo && summaryData.cpuInfo.usage && summaryData.cpuInfo.usage.average() + '%' || 'UNKNOWN'",
                            'popover' => array( "CPU Load", "js:cpuInfoText"),
                        ), true ) 
        ) );

        $row->addCol( array( 
            'sizes'=>$sizes, 
            'content'=> $this->widget('ButPro', array( 
                            'title' => 'STORAGE', 
                            'text' => "js: Math.round( summaryData.storage.used() / summaryData.storage.tot() *100 ) + '%'", 
                            'alert'=> "js: summaryData.storage.free() / summaryData.storage.tot() < 0.05", 
                            'popover' => array( "Storage", "js:storageInfoText"), 
                        ), true ),                    
        ) );   
        if ( 0 && yii::app()->user->ikebanaBetaFeatureRemove ){
            $displayStr = $screenInfo->resolution ."@".$screenInfo->refresh."Hz";
        } else {
            $displayStr = "js: summaryData.screenInfo.text";
        }
        $row->addCol( array( 
            'sizes'=>$sizes, 
            'offsets' => array( 'xl' => 4 ),
            'pulls' => array( 'xl' => 4 ),
            'content'=> $this->widget('ButPro', array( 
                            'title' => 'DISPLAY', 
                            'text' => $displayStr,
                            'alert' => "js: summaryData.screenInfo.alert",
                            'popover' => array( "Display", "js:displayInfoText"),                                    
                        ), true ) 
        ) ); 

        $row->addCol( array( 'sizes'=>$sizes, 
                             'visibles' => array( 'sm'=>$verticalSnapshot?null:true, 'lg'=> $verticalSnapshot?true:null, 'xl'=>$verticalSnapshot?false:null ), 
                             'content'=> "" ) );

        $networkPopover = false;
        if ( $basicInfo->isZeroConf() ){
            $network = "LIMITED";
            $networkAlert = true;
            $networkPopover = array( "Limited network conectivity",
                "This device is using a Zeroconf IP address (link-local) and thus it has limited network conectivity." ." ".
                "Possible causes are: no DHCP server is available or the assigned static IP address is already in use." .
                $ipConfigLink
            );
        } else {
            $networkAlert = "js:ipChecked().executed && !ipChecked().success";
            $network = "js: !ipChecked().executed?'CHECKING...':(!ipChecked().success?'NO INTERNET':'OK')";               
            $networkPopover = array( "Network", "js:networkInfoText" );
        }

        $row->addCol( array( 
            'sizes'=>$sizes, 
            'pulls' => array( 'xl' => 4 ),            
            'content'=> $this->widget('ButPro', array( 
                            'title' => 'NETWORK', 
                            'text' => $network,
                            'alert' => $networkAlert,
                            'popover' => $networkPopover
                        ), true ) 
        ) );

        $firmwareAlert = "js: firmwareInfo().executed && ( !firmwareInfo().success || firmwareInfo().updates > 0)";

        $row->addCol( array( 
            'sizes'=>$sizes, 
            'pulls' => array( 'xl' => 4 ),            
            'content'=> $this->widget('ButPro', array( 
                            'title' => 'FIRMWARE', 
                            'text' => "js: summaryData.firmware",
                            'alert' => $firmwareAlert,
                            'popover' => array( "Firmware", "js:firwareInfoTextButton"),
                            //'link' => array( 'operations' ,'#'=> 'Firmware' ) 
                        ), true ) 
        ) );
    $this->endWidget();
$this->endWidget();
$Info = ob_get_contents ( );
ob_end_clean();


$main = $this->beginWidget('BsRow');

    if ( $verticalSnapshot ){
        $main->addCol( array(
            'sizes' => array( 'sm' => 7, 'md'=> 6, 'xl' => 3 ),
            'pushs' => array( 'sm' => 5, 'md'=> 6, 'xl' => 9 ),        
            'content' => $Snapshot
        ));
        $main->addCol( array(
            'sizes' => array( 'sm' => 5, 'md' => 6, 'xl' => 9 ),   
            'pulls' => array( 'sm' => 7, 'md' => 6, 'xl' => 3 ), 
            'content' => $Info
        ));
        $main->addCol( array(
            'sizes' => array( 'sm' => 12 ), 
            'options' => array( 'class' => 'clearfix' ),   
            'visibles' => array( 'sm' => true ), 
            'content' => ""
        ));
    } else {
        $main->addCol( array(
            'sizes' => array( 'md' => 12 ),       
            'content' => $Info
        ));

        $main->addCol( array(
            'sizes' => array( 'md' => 6 ),
            'pushs' => array( 'md' => 6 ),        
            'content' => $Snapshot
        ));
    }
    $main->addCol( array(
        'sizes' => array( 'sm' => 12, 'md' => 6, 'xl' => $verticalSnapshot?4:6 ),
        'pulls' => array( 'md' => 6, 'xl' => $verticalSnapshot?3:6 ),        
        'content' => $Settings
    ));    

    $main->addCol( array(
        'sizes' => array( 'sm' => 12, 'md' => 6, 'xl' => $verticalSnapshot?5:6 ),
        'pulls' => array( 'md' => 6, 'xl' => $verticalSnapshot?3:6 ),        
        'content' => $Links
    ));
    
$this->endWidget();

$this->renderPartial('wizard', array( 
    'timeSettings' => $timeSettings,
    'playerConfig' => $playerConfig,
    'screenSettings' => $screenSettings,
    'user' => $user,
    'manual' => !$wizard
) );

?>
<script type="text/javascript" >
//<![CDATA[
extendModel( yiiModel );
extendModelFirmware( yiiModel, <?php echo CJavaScript::encode( $configFirmware ); ?> );


yiiModel.firwareInfoTextButton = ko.computed( function() {
   var txt = "<p>" + yiiModel.firwareInfoText() + "</p>";
   var info = yiiModel.firmwareInfo();
<?php if (Yii::app()->user->checkAccess('useControlCenter')) {?>
   if ( info.executed && info.success && info.updates>0 ){
       txt += "<button onclick=\"firmwareUpdateAction()\" class=\"btn btn-default pull-right\">Update Now</button>";
   }
<?php } ?>
    return txt;
});

yiiModel.summaryData =  ko.mapping.fromJS( <?php echo CJavaScript::encode( $data );?>);

yiiModel.displayInfoText = ko.computed( function() {
<?php
$screenData = array(
    array( 'label'=>"Type", 'value' =>  "{type}" ),
    array( 'label'=>"Power", 'value' =>  "{power}" )
);
echo "    var template = ".CJavaScript::encode( 
    $this->widget('DataTable', array( 'data' => $screenData ), true)
    ) .";\n";
echo "    var link = ".CJavaScript::encode($displayLink).";\n";
echo "    var link2 = ".CJavaScript::encode($displayPowerLink).";\n";
?>
    if ( yiiModel.summaryData.screenInfo && yiiModel.summaryData.screenInfo.type ) {
        var txt = template.replace("{power}", yiiModel.summaryData.screenInfo.text())
                       .replace("{type}", yiiModel.summaryData.screenInfo.type());
        if ( yiiModel.summaryData.screenInfo.desc() !== "" ){
            txt += yiiModel.summaryData.screenInfo.desc() + "<br/>" ;
            txt += link2 + "<br/>";
        }
        txt += link;
        return txt;
    } else {
        return <?php if ( !yii::app()->user->ikebanaBetaFeatureRemove ) {?>"No display detected<br/>" + <?php } ?>  link;
   }
});
yiiModel.storageInfoText = ko.computed( function() {
<?php
$storageData = array(
    array( 'label'=>"Free", 'value' =>  "{free} {unit}" ),
    array( 'label'=>"Used", 'value' =>  "{used} {unit}" ),
    array( 'label'=>"Total", 'value' =>  "{total} {unit}" )
);
echo "    var template = ".CJavaScript::encode( 
    $this->widget('DataTable', array( 'data' => $storageData ), true)
    ) .";\n";
echo "    var link = ".CJavaScript::encode( $storageLink ).";\n";
?>
    if ( yiiModel.summaryData.storage ) {
        return template.replace("{free}", yiiModel.summaryData.storage.free())
                       .replace("{used}", yiiModel.summaryData.storage.used())
                       .replace("{total}", yiiModel.summaryData.storage.tot())
                       .replace(/{unit}/g, yiiModel.summaryData.storage.unit())+
               link;    
   } else {
       return link;
   }
});
yiiModel.cpuInfoText = ko.computed( function() {
<?php
$cpuData = array(
    array( 'label'=>"CPU Average", 'value' =>  "{cpu}%<br/>ARM: {arm}%, DSP: {dsp}%" ),
    array( 'label'=>"Peak rendering time", 'value' =>  "{time}ms" ),
    array( 'label'=>"Dropped frames", 'value' =>  "{drop}%<br/>FPS: {fps}%, Peak: {peaks}%" )
);
echo "    var template = ".CJavaScript::encode( 
    $this->widget('DataTable', array( 'data' => $cpuData ), true)
    ) .";\n";
echo "    var link = ".CJavaScript::encode( $logLink ).";\n";
?>
    if ( yiiModel.summaryData.cpuInfo && yiiModel.summaryData.cpuInfo.usage ) {
        return template.replace("{cpu}", yiiModel.summaryData.cpuInfo.usage.average())
                       .replace("{arm}", yiiModel.summaryData.cpuInfo.usage.arm())
                       .replace("{dsp}", yiiModel.summaryData.cpuInfo.usage.dsp())
                       .replace("{time}", yiiModel.summaryData.cpuInfo.peak())
                       .replace("{drop}", yiiModel.summaryData.cpuInfo.dropFPS.average() + yiiModel.summaryData.cpuInfo.dropPeak.average() )
                       .replace("{fps}", yiiModel.summaryData.cpuInfo.dropFPS.average())
                       .replace("{peaks}", yiiModel.summaryData.cpuInfo.dropPeak.average())+
               link;    
   } else {
       return link;
   }
});
yiiModel.networkInfoText = ko.computed( function() {
<?php
$ipData = array(
    array( 'label'=>"IP", 'value' =>  $basicInfo->getIp() ),
    array( 'label'=>"IPv6", 'value' =>  $basicInfo->getIpv6() ),
    array( 'label'=>"Hostname", 'value' =>  $basicInfo->getHostname() .".local" ),
    array( 'label'=>"MAC", 'value' =>  $basicInfo->getMAC() )
);
echo "    var template = ".CJavaScript::encode( 
    $this->widget('DataTable', array( 'data' => $ipData ), true)
    ) .";\n";
echo "    var link = ".CJavaScript::encode( $ipConfigLink ).";\n";
?>
   if ( !yiiModel.ipChecked().success ) {
        return template + "<p>" + yiiModel.ipChecked().reason + "</p>" + link;    
   } else {
       return template + link;
   }
});
$(function() {
    function updateTime(){    
        var now = moment();
        var target = moment( now + delta);
        yiiModel.summaryData.time( target.format("D MMM YYYY H:mm:ss") );
        //$('#LocalTime').text( target.format("D MMM YYYY H:mm:ss") );
        setTimeout(updateTime, 1000);
    }
    var now = moment();
    var initTime = moment( yiiModel.summaryData.time(), "D MMM YYYY H:mm:ss");
    var delta = initTime - now;
    var timer = setTimeout(updateTime, 1000);

    yiiModel.checkFirmware( 'base', false, true );    
    $('[data-toggle="popover"]').popover( {
        html: true,
        container: 'body'
    });
    function updateData(){
        $.ajax( <?php echo CJavaScript::encode($summary);?> ).done( function( response ) {
            ko.mapping.fromJS(response.data, yiiModel.summaryData);
            now = moment();
            initTime = moment( yiiModel.summaryData.time(), "D MMM YYYY H:mm:ss");
            delta = initTime - now;
            clearTimeout(timer);
            timer = setTimeout(updateTime, 1000);
            setTimeout(updateData, 1*60*1000);
        });
    }
    setTimeout(updateData, 1*60*1000);
    
    $('#CockpitMonitor').hide();
    $.ajax({url: <?php echo CJavaScript::encode( $cockpitHost.$cockpitNewPlayer);?>, 
        type: "GET",
        crossDomain: true, 
        xhrFields: { withCredentials: true } })      
     .done(function( data, textStatus, jqXHR ){         
         if ( data.url ){
            $('#CockpitUri').val( data.url );
            $('#CockpitMonitor').show();
            $('#GoSystem').hide();
         }
     });
});
function firmwareUpdateAction( ev ){
    location.assign( <?php echo "'".CHtml::normalizeUrl(array("/main/firmware"))."'"; ?> );
}

//]]></script>