<?php

/*
 * This script is run on package update, from a boot script on the
 * first reboot after an update, *before* the HTTP service is started.
 * Note that stdout and stderr go to the console, not a browser. 
 *
 * Put here any complex operations that are required after a package
 * update. As much as possible the operations should only be performed
 * if necessary, so that minimal system modification is done.
 *
 * The operations in here should also be idempotent, running them more
 * than once should have no ill effects, as this may happen under some
 * circusmtances after an update (think power loss at just the wrong
 * moment).
 * 
 * Conditions to run the scrip:
 * A) Following files do not exists
 *  /var/lib/spxmanage/ui/ui.db
 *  /srv/raperca/interface/protected/assets.db
 * B) Following file exists:
 *  /var/spool/spxmanage/run-update-jobs
 * 
 */

include_once( __DIR__.'/runYiic.php' );
$script = basename( __FILE__ );
// make sure all the errors are also pushed to stderr
ini_set('display_errors', 'stderr');

syslog(LOG_NOTICE, 'running migration for main db' );
runYiic( array( 'yiic.php', 'migrate', 'up', '--interactive=0' ), $script, true);

syslog(LOG_NOTICE, 'running migration for assets db' );
runYiic( array( 'yiic.php', 'migrate', 'up', '--interactive=0', '--migrationPath=application.migrations_assets', '--connectionID=dbassets' ), $script, false);

syslog(LOG_NOTICE, 'updating assets' );
runYiic( array( 'yiic.php', 'assets', 'update' ), $script, false);